"""
批量数据库连接管理器模块
专门用于管理大量配置相似的数据库连接（临时性配置）

主要特性：
- 基于模板的批量连接配置
- 并发执行数据库操作
- 批量表结构升级支持
- 完善的错误处理和回滚机制
- IP范围自动生成工具

使用示例：
    >>> from db_connector_tool import BatchDatabaseManager, generate_ip_range
    >>>
    >>> # 创建批量管理器
    >>> batch_manager = BatchDatabaseManager("my_app")
    >>>
    >>> # 设置基础配置模板
    >>> base_config = {
    ...     "type": "mysql",
    ...     "port": 3306,
    ...     "username": "admin",
    ...     "password": "password",
    ...     "database": "user_db"
    ... }
    >>> batch_manager.set_base_config(base_config)
    >>>
    >>> # 批量添加连接
    >>> ip_list = generate_ip_range("192.168.1.100", 50)
    >>> results = batch_manager.add_batch_connections(ip_list)
"""

import ipaddress
import threading
from concurrent.futures import ThreadPoolExecutor, as_completed
from typing import Any, Dict, List, Optional

from .core.connections import DatabaseManager
from .core.exceptions import QueryError
from .utils.logging_utils import get_logger
from .utils.path_utils import PathHelper

# 获取模块级别的日志记录器
logger = get_logger(__name__)


class BatchDatabaseManager:
    """
    批量数据库连接管理器（临时性配置）

    适用于管理大量配置相似的数据库连接，支持批量操作和并发执行。
    使用独立的临时配置文件，避免与主配置文件冲突。

    Attributes:
        base_config (Dict[str, Any]): 基础配置模板
        db_manager (DatabaseManager): 数据库管理器实例
        connection_pool (Dict[str, Dict]): 连接池字典
        _lock (threading.RLock): 线程安全锁
    """

    def __init__(self, temp_config_suffix: str = "batch"):
        """
        初始化批量数据库管理器（临时性配置）

        Args:
            temp_config_suffix: 临时配置后缀，用于创建独立的配置文件
                            例如："batch" -> "connections_batch.toml"
                            不能为"connections.toml"或空字符串

        Raises:
            ValueError: 当配置后缀无效时
        """
        if not temp_config_suffix or temp_config_suffix.strip() == "":
            raise ValueError("临时配置后缀不能为空")

        if temp_config_suffix in ["connections", "connections.toml"]:
            raise ValueError("临时配置后缀不能为'connections'，以避免与主配置文件冲突")

        # 使用主应用名称，但创建独立的临时配置文件
        self.app_name = "db_connector_tool"
        self.temp_config_suffix = temp_config_suffix
        self.temp_config_file = f"connections_{temp_config_suffix}.toml"

        self.db_manager = DatabaseManager(self.app_name, self.temp_config_file)

        self.base_config: Optional[Dict[str, Any]] = None
        self._connection_names = []
        self._lock = threading.RLock()
        self._is_cleaned = False

        logger.info(f"批量管理器初始化完成 - 临时配置文件: {self.temp_config_file}")

    def set_base_config(self, config: Dict[str, Any]) -> None:
        """
        设置基础配置模板

        Args:
            config: 基础数据库配置，包含除IP外的所有参数
        """
        self.base_config = config.copy()
        # 移除IP字段，因为IP是动态变化的
        if "host" in self.base_config:
            del self.base_config["host"]

        logger.info("基础配置模板设置完成")

    def add_batch_connections(
        self, ip_list: List[str], connection_prefix: str = "db"
    ) -> Dict[str, bool]:
        """
        批量添加数据库连接

        如果连接名称已存在，会先删除旧连接再添加新连接。

        Args:
            ip_list: IP地址列表，如 ["192.168.1.100", "192.168.1.101", ...]
            connection_prefix: 连接名称前缀，实际名称为 {prefix}_{index}

        Returns:
            Dict[str, bool]: 每个IP的连接添加结果
        """
        if not self.base_config:
            raise ValueError("请先设置基础配置模板")

        results = {}
        success_count = 0

        for index, ip in enumerate(ip_list):
            connection_name = f"{connection_prefix}_{index:03d}"

            # 创建具体配置
            specific_config = self.base_config.copy()
            specific_config["host"] = ip

            try:
                connection_names = self.db_manager.list_connections()
                # 检查连接是否已存在，如果存在则先删除
                if connection_name in connection_names:
                    logger.warning(f"连接 {connection_name} 已存在，执行覆盖操作")
                    self._remove_existing_connection(connection_name)

                # 添加新连接
                self.db_manager.add_connection(connection_name, specific_config)

                # 添加到连接名称列表
                with self._lock:
                    if connection_name not in self._connection_names:
                        self._connection_names.append(connection_name)

                results[ip] = True
                success_count += 1
                logger.info(f"添加连接: {connection_name} -> {ip}")
            except Exception as e:
                results[ip] = False
                logger.error(f"添加连接失败 {ip}: {e}")

        logger.info(f"批量添加完成: {success_count}/{len(ip_list)} 个连接成功")
        return results

    def cleanup(self) -> None:
        """
        清理批量管理器所有资源

        安全关闭所有数据库连接，清理临时配置文件，释放所有资源。

        Raises:
            Exception: 当清理过程中发生严重错误时

        Example:
            >>> batch_manager.cleanup()
        """
        if self._is_cleaned:
            logger.debug("批量管理器已清理，无需重复操作")
            return

        try:
            logger.info("开始清理批量管理器资源")

            # 1. 关闭所有数据库连接
            connection_names = self._get_all_connection_names()
            if connection_names:
                logger.info(f"关闭 {len(connection_names)} 个数据库连接")
                for conn_name in connection_names:
                    try:
                        self.db_manager.close_connection(conn_name)
                        logger.debug(f"连接 {conn_name} 已关闭")
                    except Exception as e:
                        logger.warning(f"关闭连接 {conn_name} 失败: {str(e)}")

            # 2. 清理连接名称列表
            with self._lock:
                self._connection_names.clear()
                logger.debug("连接名称列表已清空")

            # 3. 清理临时配置文件
            try:
                config_path = (
                    PathHelper.get_user_config_dir(self.app_name)
                    / self.temp_config_file
                )
                if config_path.exists():
                    config_path.unlink()
                    logger.info(f"临时配置文件已删除: {config_path}")
                else:
                    logger.debug("临时配置文件不存在，无需删除")
            except Exception as e:
                logger.warning(f"删除临时配置文件失败: {str(e)}")

            # 4. 清理基础配置
            self.base_config = None

            self._is_cleaned = True
            logger.info("批量管理器资源清理完成")

        except Exception as e:
            logger.error(f"批量管理器资源清理失败: {str(e)}")
            raise

    def __enter__(self):
        """上下文管理器入口"""
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """上下文管理器出口，确保资源清理"""
        self.cleanup()

    def __del__(self):
        """析构函数，确保资源清理"""
        if not self._is_cleaned:
            try:
                self.cleanup()
            except Exception:
                # 析构函数中忽略异常
                pass

    def _remove_existing_connection(self, connection_name: str) -> None:
        """
        安全删除已存在的连接配置

        Args:
            connection_name: 要删除的连接名称

        Note:
            确保连接被完全清理，包括关闭连接和清理配置
        """
        with self._lock:
            try:
                # 1. 先关闭连接
                try:
                    self.db_manager.close_connection(connection_name)
                    logger.debug(f"连接 {connection_name} 已关闭")
                except Exception as close_error:
                    logger.warning(
                        f"关闭连接 {connection_name} 失败: {str(close_error)}"
                    )

                # 2. 从数据库管理器中删除连接配置
                try:
                    self.db_manager.remove_connection(connection_name)
                    logger.debug(f"已从数据库管理器中删除连接配置: {connection_name}")
                except Exception as remove_error:
                    logger.warning(
                        f"从数据库管理器删除连接配置失败 {connection_name}: {str(remove_error)}"
                    )

                # 3. 从连接名称列表中移除
                if connection_name in self._connection_names:
                    self._connection_names.remove(connection_name)
                    logger.debug(f"已从连接名称列表中移除: {connection_name}")

                logger.info(f"连接 {connection_name} 删除完成")

            except Exception as e:
                logger.error(f"删除连接 {connection_name} 时发生严重错误: {str(e)}")
                raise

    def test_batch_connections(self, max_workers: int = 10) -> Dict[str, bool]:
        """
        批量测试数据库连接

        Args:
            max_workers: 最大并发线程数

        Returns:
            Dict[str, bool]: 每个连接的测试结果
        """
        connection_names = self._get_all_connection_names()
        if not connection_names:
            logger.warning("没有可测试的连接")
            return {}

        results = {}
        success_count = 0

        def test_single_connection(conn_name):
            try:
                is_connected = self.db_manager.test_connection(conn_name)
                return conn_name, is_connected
            except Exception as e:
                logger.error(f"测试连接 {conn_name} 失败: {e}")
                return conn_name, False

        # 使用线程池并发测试
        with ThreadPoolExecutor(max_workers=max_workers) as executor:
            future_to_conn = {
                executor.submit(test_single_connection, conn_name): conn_name
                for conn_name in connection_names
            }

            for future in as_completed(future_to_conn):
                conn_name, result = future.result()
                results[conn_name] = result
                if result:
                    success_count += 1
                status = "正常" if result else "异常"
                logger.info(f"{status} - {conn_name}")

        logger.info(f"批量测试完成: {success_count}/{len(connection_names)} 个连接正常")
        return results

    def execute_batch_query(
        self, sql: str, params: Optional[Dict] = None, max_workers: int = 5
    ) -> Dict[str, Any]:
        """
        批量执行SQL查询

        Args:
            sql: 要执行的SQL语句
            params: 查询参数
            max_workers: 最大并发线程数

        Returns:
            Dict[str, Any]: 每个连接的执行结果
        """
        connection_names = self._get_all_connection_names()
        if not connection_names:
            logger.warning("没有可执行查询的连接")
            return {}

        results = {}

        def execute_single_query(conn_name):
            try:
                result = self.db_manager.execute_query(conn_name, sql, params or {})
                return conn_name, {
                    "success": True,
                    "data": result,
                    "count": len(result),
                }
            except QueryError as e:
                return conn_name, {"success": False, "error": str(e), "data": []}
            except Exception as e:
                return conn_name, {"success": False, "error": str(e), "data": []}

        with ThreadPoolExecutor(max_workers=max_workers) as executor:
            future_to_conn = {
                executor.submit(execute_single_query, conn_name): conn_name
                for conn_name in connection_names
            }

            for future in as_completed(future_to_conn):
                conn_name, result = future.result()
                results[conn_name] = result

        success_count = sum(1 for r in results.values() if r["success"])
        logger.info(f"批量查询完成: {success_count}/{len(connection_names)} 个连接成功")
        return results

    def upgrade_table_structure(
        self,
        upgrade_sqls: List[str],
        rollback_sqls: Optional[List[str]] = None,
        max_workers: int = 3,
    ) -> Dict[str, Dict]:
        """
        批量升级表结构（支持事务和回滚）

        Args:
            upgrade_sqls: 升级SQL语句列表
            rollback_sqls: 回滚SQL语句列表（可选）
            max_workers: 最大并发线程数（建议较小，避免对数据库造成过大压力）

        Returns:
            Dict[str, Dict]: 每个连接的升级结果
        """
        connection_names = self._get_all_connection_names()
        if not connection_names:
            logger.warning("没有可升级的连接")
            return {}

        results = {}

        def upgrade_single_database(conn_name):
            try:
                execution_results = []

                for sql in upgrade_sqls:
                    try:
                        result = self.db_manager.execute_query(conn_name, sql)
                        execution_results.append(
                            {
                                "sql": sql,
                                "success": True,
                                "affected_rows": len(result) if result else 0,
                            }
                        )
                    except QueryError as e:
                        execution_results.append(
                            {"sql": sql, "success": False, "error": str(e)}
                        )
                        # 如果某条SQL失败，尝试回滚
                        if rollback_sqls:
                            self._execute_rollback(conn_name, rollback_sqls)
                        break

                return conn_name, {
                    "success": all(r["success"] for r in execution_results),
                    "executions": execution_results,
                }

            except Exception as e:
                return conn_name, {"success": False, "error": str(e), "executions": []}

        with ThreadPoolExecutor(max_workers=max_workers) as executor:
            future_to_conn = {
                executor.submit(upgrade_single_database, conn_name): conn_name
                for conn_name in connection_names
            }

            for future in as_completed(future_to_conn):
                conn_name, result = future.result()
                results[conn_name] = result

        success_count = sum(1 for r in results.values() if r["success"])
        logger.info(f"批量升级完成: {success_count}/{len(connection_names)} 个连接成功")
        return results

    def _get_all_connection_names(self) -> List[str]:
        """获取所有连接名称"""
        with self._lock:
            return self._connection_names.copy()

    def _execute_rollback(self, conn_name: str, rollback_sqls: List[str]) -> None:
        """执行回滚操作"""
        for sql in rollback_sqls:
            try:
                self.db_manager.execute_query(conn_name, sql)
            except Exception as e:
                logger.warning(f"回滚执行失败 {conn_name}: {e}")

    def get_connection_stats(self) -> Dict[str, Any]:
        """获取连接统计信息"""
        stats = {}
        connection_names = self._get_all_connection_names()

        for conn_name in connection_names:
            try:
                stats[conn_name] = self.db_manager.get_connection_info(conn_name)
            except Exception as e:
                stats[conn_name] = {"error": str(e)}

        return stats

    def close_all_connections(self) -> None:
        """关闭所有连接"""
        self.db_manager.close_all_connections()
        logger.info("所有批量连接已关闭")


def generate_ip_range(base_ip: str, count: int) -> List[str]:
    """
    生成连续的IP地址列表

    Args:
        base_ip: 基础IP地址，如 "192.168.1.100"
        count: 生成的IP数量

    Returns:
        List[str]: IP地址列表
    """
    base = ipaddress.IPv4Address(base_ip)
    return [str(base + i) for i in range(count)]


def cleanup_temp_configs(app_name: str = "db_connector_tool") -> None:
    """
    清理所有临时批量配置文件

    Args:
        app_name: 应用名称，默认为"db_connector_tool"
    """
    config_dir = PathHelper.get_user_config_dir(app_name)

    if config_dir.exists():
        # 查找所有临时配置文件
        temp_files = list(config_dir.glob("connections_*.toml"))

        for temp_file in temp_files:
            try:
                temp_file.unlink()
                logger.info(f"清理临时配置文件: {temp_file}")
            except Exception as e:
                logger.warning(f"清理临时配置文件失败 {temp_file}: {e}")

        logger.info(f"共清理 {len(temp_files)} 个临时配置文件")



"""
配置管理模块

使用 TOML 格式进行配置管理，提供数据库连接配置的加密存储和管理功能。
支持配置文件的创建、加载、保存，以及连接配置的增删改查操作。

特性：
- 基于 TOML 格式的配置文件，易于阅读和编辑
- 全字段加密：所有连接配置字段自动加密存储
- 数据类型保持：序列化/反序列化机制保留原始数据类型
- 版本兼容性检查：支持多版本配置格式
- 自动备份功能：支持配置文件备份和恢复

安全特性：
- 使用加密管理器保护敏感信息
- 密钥文件独立存储，增强安全性
- 配置文件完整性验证
"""

import json
import platform
import shutil
import stat
import subprocess
import tomllib
from datetime import datetime
from pathlib import Path
from typing import Any, Dict, List, Optional

import tomli_w

from ..utils.logging_utils import get_logger
from ..utils.path_utils import PathHelper
from .crypto import CryptoManager
from .exceptions import ConfigError

# 获取模块级别的日志记录器
logger = get_logger(__name__)

# 配置文件创建时的初始时间戳（固定值）
CONFIG_CREATION_TIMESTAMP = datetime.now().astimezone().isoformat()

# 版本号递增规则
VERSION_INCREMENT_ON_CHANGE = True  # 每次配置变更时自动递增版本号

# 错误消息常量
ERROR_EMPTY_CONNECTION_NAME = "连接名称不能为空且必须是字符串"
ERROR_INVALID_CONFIG_DICT = "连接配置不能为空且必须是字典"
ERROR_VERSION_MAJOR_CHANGE = "版本号递增导致第一位发生变化，变更过于频繁，请手动检查"


class ConfigManager:
    """
    配置管理器类

    管理数据库连接配置的加密存储，使用TOML格式配置文件。
    提供连接配置的增删改查功能，所有敏感信息都会自动加密。

    Attributes:
        app_name (str): 应用名称，用于确定配置目录
        config_file (str): 配置文件名
        config_dir (Path): 配置目录路径
        config_path (Path): 完整配置文件路径
        crypto (Optional[CryptoManager]): 加密管理器实例
    """

    def __init__(
        self, app_name: str = "db_connector_tool", config_file: str = "connections.toml"
    ) -> None:
        """
        初始化配置管理器

        Args:
            app_name: 应用名称，用于确定配置目录
            config_file: 配置文件名，默认为"connections.toml"

        Raises:
            ConfigError: 当配置文件初始化失败时

        Example:
            >>> config_manager = ConfigManager("my_app", "database.toml")
        """
        self.app_name = app_name
        self.config_file = config_file
        self.config_dir = PathHelper.get_user_config_dir(app_name)
        self.config_path = self.config_dir / config_file
        self.crypto: Optional[CryptoManager] = None
        self._ensure_config_exists()

    def _ensure_config_exists(self) -> None:
        """
        确保配置文件存在，如果不存在则创建默认配置

        Raises:
            ConfigError: 当配置文件创建或初始化失败时
        """
        try:
            if not self.config_path.exists():
                self._create_default_config()
            self._load_or_create_crypto_key_secure()
            logger.debug(f"配置文件就绪: {self.config_path}")
        except Exception as e:
            logger.error(f"初始化配置文件失败: {str(e)}")
            raise ConfigError(f"配置文件初始化失败: {str(e)}") from e

    def _create_default_config(self) -> None:
        """
        创建默认配置文件结构

        Raises:
            ConfigError: 当默认配置创建失败时
        """
        default_config = {
            "version": "1.0.0",
            "app_name": self.app_name,
            "connections": {},
            "metadata": {
                "created": CONFIG_CREATION_TIMESTAMP,
                "last_modified": CONFIG_CREATION_TIMESTAMP,
                "config_file": str(self.config_path),
            },
        }
        self._save_config(default_config)
        logger.info(f"创建默认配置文件: {self.config_path}")

    def _save_config(self, config: Dict[str, Any]) -> None:
        """
        保存配置文件

        Args:
            config: 要保存的配置字典

        Raises:
            ConfigError: 当配置文件保存失败时

        Process:
            1. 更新最后修改时间
            2. 验证配置结构
            3. 保存为TOML格式
        """
        try:
            # 更新最后修改时间
            current_time = datetime.now().astimezone().isoformat()
            config["metadata"]["last_modified"] = current_time

            # 验证配置结构
            self._validate_config(config)

            with open(self.config_path, "wb") as f:
                f.write(tomli_w.dumps(config).encode("utf-8"))

            logger.debug(f"配置文件已保存: {self.config_path}")

        except Exception as e:
            logger.error(f"保存配置文件失败: {str(e)}")
            raise ConfigError(f"配置文件保存失败: {str(e)}") from e

    def _validate_config(self, config: Dict[str, Any]) -> None:
        """
        验证配置文件结构

        Args:
            config: 要验证的配置字典

        Raises:
            ConfigError: 当配置结构无效时

        Validation Rules:
            - 必须包含 version, app_name, connections, metadata 字段
            - 版本必须在支持的版本列表中
        """
        required_fields = ["version", "app_name", "connections", "metadata"]
        for field in required_fields:
            if field not in config:
                raise ConfigError(f"配置文件缺少必需字段: {field}")

    def _load_or_create_crypto_key_secure(self) -> None:
        """
        使用操作系统密钥存储服务加载或创建加密密钥

        Raises:
            ConfigError: 当密钥加载或创建失败时
        """
        try:
            # 尝试使用keyring库（如果可用）
            try:
                import keyring

                service_name = f"{self.app_name}_crypto"
                username = "master_key"

                # 尝试从密钥环获取密钥
                stored_key = keyring.get_password(service_name, username)

                if stored_key:
                    # 解析存储的密钥数据
                    key_data = json.loads(stored_key)
                    self.crypto = CryptoManager.from_saved_key(
                        key_data["password"], key_data["salt"]
                    )
                    logger.debug("从操作系统密钥存储加载密钥成功")
                else:
                    # 创建新密钥并存储
                    self.crypto = CryptoManager()
                    key_data = self.crypto.get_key_info()
                    keyring.set_password(service_name, username, json.dumps(key_data))
                    logger.info("新加密密钥已安全存储到操作系统密钥环")

            except ImportError:
                # keyring不可用，回退到文件权限方案
                logger.warning("keyring库不可用，使用文件权限保护方案")
                self._load_or_create_crypto_key()

        except Exception as e:
            logger.error(f"安全密钥管理失败: {str(e)}")
            raise ConfigError(f"安全密钥管理失败: {str(e)}") from e

    def _load_or_create_crypto_key(self) -> None:
        """
        加载或创建加密密钥

        Raises:
            ConfigError: 当密钥加载或创建失败时
        """
        key_file = self.config_dir / "encryption.key"

        if key_file.exists():
            # 加载现有密钥
            try:
                # 设置文件权限为仅所有者可读写
                self._secure_file_permissions(key_file)

                with open(key_file, "r", encoding="utf-8") as f:
                    key_data = tomllib.loads(f.read())

                if "password" not in key_data or "salt" not in key_data:
                    raise ConfigError("密钥文件格式无效")

                self.crypto = CryptoManager.from_saved_key(
                    key_data["password"], key_data["salt"]
                )
                logger.debug("加密密钥加载成功")

            except Exception as e:
                logger.error(f"加载加密密钥失败: {str(e)}")
                raise ConfigError(f"加密密钥加载失败: {str(e)}") from e
        else:
            # 创建新密钥
            try:
                self.crypto = CryptoManager()
                key_data = self.crypto.get_key_info()

                # 先写入文件，然后设置安全权限
                with open(key_file, "w", encoding="utf-8") as f:
                    f.write(tomli_w.dumps(key_data))

                # 设置文件权限为仅所有者可读写
                self._secure_file_permissions(key_file)

                logger.info("新加密密钥创建成功")

            except Exception as e:
                logger.error(f"创建加密密钥失败: {str(e)}")
                raise ConfigError(f"加密密钥创建失败: {str(e)}") from e

    def _secure_file_permissions(self, file_path: Path) -> None:
        """
        设置文件安全权限，仅允许所有者读写

        Args:
            file_path: 文件路径

        Raises:
            ConfigError: 当权限设置失败时
        """
        try:
            system = platform.system().lower()

            if system == "windows":
                # Windows系统：使用icacls设置权限
                try:
                    # 移除所有用户的权限
                    subprocess.run(
                        ["icacls", str(file_path), "/reset"],
                        check=True,
                        capture_output=True,
                    )
                    # 仅当前用户完全控制
                    subprocess.run(
                        ["icacls", str(file_path), "/grant", f"%username%:F"],
                        check=True,
                        capture_output=True,
                    )
                    # 拒绝其他所有用户访问
                    subprocess.run(
                        ["icacls", str(file_path), "/inheritance:r"],
                        check=True,
                        capture_output=True,
                    )
                except (subprocess.CalledProcessError, FileNotFoundError):
                    # 如果icacls不可用，使用基本的文件属性
                    file_path.chmod(stat.S_IREAD | stat.S_IWRITE)
            else:
                # Unix-like系统：设置600权限（仅所有者可读写）
                file_path.chmod(stat.S_IREAD | stat.S_IWRITE)

            logger.debug("设置文件权限成功")

        except Exception as e:
            logger.warning(f"设置文件权限失败 {file_path}: {str(e)}")
            # 权限设置失败不应阻止程序运行，但记录警告

    def add_connection(self, name: str, connection_config: Dict[str, Any]) -> None:
        """
        添加数据库连接配置

        Args:
            name: 连接名称（唯一标识符）
            connection_config: 连接配置字典

        Raises:
            ConfigError: 当连接已存在或添加失败时
            ValueError: 当连接名称为空或配置无效时

        Security Note:
            - 所有配置字段都会自动加密存储
            - 连接名称作为唯一标识符，不能重复

        Example:
            >>> config = {
            ...     "host": "localhost",
            ...     "port": 5432,
            ...     "username": "admin",
            ...     "password": "secret"
            ... }
            >>> config_manager.add_connection("postgres_db", config)
        """
        if not name or not isinstance(name, str):
            raise ValueError(ERROR_EMPTY_CONNECTION_NAME)

        if not connection_config or not isinstance(connection_config, dict):
            raise ValueError(ERROR_INVALID_CONFIG_DICT)

        try:
            config = self._load_config()

            # 检查连接是否已存在
            if name in config["connections"]:
                raise ConfigError(f"连接配置已存在: {name}")

            # 确保加密管理器已初始化
            if self.crypto is None:
                raise ConfigError("加密管理器未初始化，无法加密敏感信息")

            # 全字段加密：加密所有连接配置字段
            encrypted_config = {}
            for key, value in connection_config.items():
                # 序列化值（保留数据类型信息）
                serialized_value = self._serialize_value(value)
                # 加密序列化后的值
                encrypted_config[key] = self.crypto.encrypt(serialized_value)

            config["connections"][name] = encrypted_config

            # 更新配置文件版本号（每次调用增加修订号）
            self._increment_config_version(config)

            self._save_config(config)
            logger.info(f"连接配置已添加: {name}")

        except Exception as e:
            logger.error(f"添加连接配置失败 {name}: {str(e)}")
            if isinstance(e, ConfigError):
                raise
            raise ConfigError(f"连接配置添加失败: {str(e)}") from e

    def _load_config(self) -> Dict[str, Any]:
        """
        加载并验证配置文件

        Returns:
            Dict[str, Any]: 配置字典

        Raises:
            ConfigError: 当配置文件格式无效或版本不支持时

        Process:
            1. 读取TOML文件
            2. 验证配置文件结构
            3. 返回配置字典
        """
        try:
            with open(self.config_path, "rb") as f:
                config = tomllib.load(f)

            # 验证配置文件结构
            self._validate_config(config)

            return config

        except tomllib.TOMLDecodeError as e:
            logger.error(f"配置文件TOML格式错误: {str(e)}")
            raise ConfigError(f"配置文件格式无效: {str(e)}") from e
        except Exception as e:
            logger.error(f"加载配置文件失败: {str(e)}")
            raise ConfigError(f"配置文件加载失败: {str(e)}") from e

    def _serialize_value(self, value: Any) -> str:
        """
        序列化值以便加密，保留数据类型信息

        Args:
            value: 要序列化的任意类型值

        Returns:
            str: JSON格式的序列化字符串

        Process:
            1. 获取值的类型名称
            2. 将值和类型信息序列化为JSON

        Example:
            >>> serialized = self._serialize_value("secret_password")
            >>> print(serialized)
            '{"type": "str", "value": "secret_password"}'
        """
        value_info = {"type": type(value).__name__, "value": value}
        return json.dumps(value_info, ensure_ascii=False)

    def _increment_config_version(self, config: Dict[str, Any]) -> None:
        """
        递增配置文件版本号

        Args:
            config: 配置字典

        Raises:
            ConfigError: 当版本号递增导致第一位发生变化时

        Process:
            1. 解析当前版本号（格式：x.y.z）
            2. 第一位保持不变，第二、三位按照10进制每次加1，满10进1
            3. 如果第一位发生变化，抛出异常
            4. 更新配置中的版本号
        """
        try:
            current_version = config["version"]
            version_parts = current_version.split(".")

            if len(version_parts) == 3:
                # 标准语义化版本号格式：x.y.z
                major, minor, patch = version_parts
                major_num = int(major)
                minor_num = int(minor)
                patch_num = int(patch)

                # 递增修订号（patch）
                patch_num += 1

                # 处理进位逻辑
                if patch_num >= 10:
                    patch_num = 0
                    minor_num += 1

                    # 处理次要版本进位
                    if minor_num >= 10:
                        minor_num = 0
                        major_num += 1

                        # 检查主版本号是否会发生重大变化（限制主版本号不超过99）
                        if major_num > 9:
                            raise ConfigError(
                                ERROR_VERSION_MAJOR_CHANGE,
                                details={
                                    "current_version": current_version,
                                    "would_become": f"{major_num}.{minor_num}.{patch_num}",
                                    "max_major_version": 9,
                                },
                            )

                new_version = f"{major_num}.{minor_num}.{patch_num}"

                # 验证新版本号格式
                if not self._is_valid_version_format(new_version):
                    raise ConfigError(
                        "版本号格式无效",
                        details={
                            "current_version": current_version,
                            "new_version": new_version,
                        },
                    )

            else:
                # 非标准版本号格式，使用默认递增逻辑
                new_version = f"{current_version}.1"
                logger.warning(
                    f"非标准版本号格式，使用默认递增: {current_version} -> {new_version}"
                )

            config["version"] = new_version
            logger.debug(f"配置文件版本号已更新: {current_version} -> {new_version}")

        except ConfigError:
            # 重新抛出ConfigError异常
            raise
        except Exception as e:
            logger.warning(f"版本号递增失败，保持原版本号: {str(e)}")
            # 如果版本号递增失败，不影响主要功能，继续使用原版本号

    def _is_valid_version_format(self, version: str) -> bool:
        """
        验证版本号格式是否有效

        Args:
            version: 版本号字符串

        Returns:
            bool: 版本号格式是否有效
        """
        try:
            parts = version.split(".")
            if len(parts) != 3:
                return False

            for part in parts:
                if not part.isdigit():
                    return False
                num = int(part)
                if num < 0:
                    return False

            return True
        except (ValueError, AttributeError):
            return False

    def remove_connection(self, name: str) -> None:
        """
        删除连接配置

        Args:
            name: 连接名称

        Raises:
            ConfigError: 当连接不存在或删除失败时

        Security Note:
            - 删除操作不可逆，建议先备份
            - 删除后加密数据将无法恢复

        Example:
            >>> config_manager.remove_connection("postgres_db")
        """
        if not name or not isinstance(name, str):
            raise ValueError(ERROR_EMPTY_CONNECTION_NAME)

        try:
            config = self._load_config()

            if name not in config["connections"]:
                raise ConfigError(f"连接配置不存在: {name}")

            del config["connections"][name]
            self._save_config(config)
            logger.info(f"连接配置已删除: {name}")

        except Exception as e:
            logger.error(f"删除连接配置失败 {name}: {str(e)}")
            if isinstance(e, ConfigError):
                raise
            raise ConfigError(f"连接配置删除失败: {str(e)}") from e

    def update_connection(self, name: str, connection_config: Dict[str, Any]) -> None:
        """
        更新连接配置

        Args:
            name: 连接名称
            connection_config: 新的连接配置字典

        Raises:
            ConfigError: 当连接不存在或更新失败时

        Process:
            1. 删除旧配置
            2. 添加新配置

        Example:
            >>> new_config = {"host": "new_host", "port": 5433}
            >>> config_manager.update_connection("postgres_db", new_config)
        """
        if not name or not isinstance(name, str):
            raise ValueError(ERROR_EMPTY_CONNECTION_NAME)

        if not connection_config or not isinstance(connection_config, dict):
            raise ValueError(ERROR_INVALID_CONFIG_DICT)

        try:
            # 先删除旧配置，再添加新配置
            self.remove_connection(name)
            self.add_connection(name, connection_config)
            logger.info(f"连接配置已更新: {name}")

        except Exception as e:
            logger.error(f"更新连接配置失败 {name}: {str(e)}")
            raise ConfigError(f"连接配置更新失败: {str(e)}") from e

    def get_connection(self, name: str) -> Dict[str, Any]:
        """
        获取数据库连接配置（自动解密）

        Args:
            name: 连接名称

        Returns:
            Dict[str, Any]: 解密后的连接配置字典

        Raises:
            ConfigError: 当连接不存在或获取失败时

        Security Note:
            - 返回的解密配置包含敏感信息，使用后应及时清理
            - 配置数据在内存中为明文状态

        Example:
            >>> config = config_manager.get_connection("postgres_db")
            >>> print(config["host"])
            "localhost"
        """
        if not name or not isinstance(name, str):
            raise ValueError(ERROR_EMPTY_CONNECTION_NAME)

        try:
            config = self._load_config()

            if name not in config["connections"]:
                raise ConfigError(f"连接配置不存在: {name}")

            connection_config = config["connections"][name].copy()

            # 确保加密管理器已初始化
            if self.crypto is None:
                raise ConfigError("加密管理器未初始化，无法解密敏感信息")

            # 全字段解密：解密所有连接配置字段并恢复数据类型
            decrypted_config = {}
            for key, encrypted_value in connection_config.items():
                # 解密值
                serialized_value = self.crypto.decrypt(encrypted_value)
                # 反序列化值（恢复原始数据类型）
                decrypted_config[key] = self._deserialize_value(serialized_value)

            logger.debug(f"连接配置已获取: {name}")
            return decrypted_config

        except Exception as e:
            logger.error(f"获取连接配置失败 {name}: {str(e)}")
            if isinstance(e, ConfigError):
                raise
            raise ConfigError(f"连接配置获取失败: {str(e)}") from e

    def _deserialize_value(self, json_str: str) -> Any:
        """
        反序列化值，恢复原始数据类型

        Args:
            json_str: JSON格式的序列化字符串

        Returns:
            Any: 反序列化后的原始值

        Raises:
            ValueError: 当JSON字符串格式无效时

        Process:
            1. 解析JSON字符串
            2. 提取原始值
            3. 如果解析失败，返回原始字符串

        Example:
            >>> original = self._deserialize_value('{"type": "str", "value": "secret"}')
            >>> print(original)
            "secret"
        """
        try:
            value_info = json.loads(json_str)
            return value_info["value"]
        except (json.JSONDecodeError, KeyError, TypeError) as e:
            logger.warning(f"反序列化失败，返回原始字符串: {str(e)}")
            # 如果JSON解析失败，尝试直接返回字符串值
            return json_str

    def list_connections(self) -> List[str]:
        """
        列出所有可用的连接名称

        Returns:
            List[str]: 连接名称列表

        Raises:
            ConfigError: 当列出连接失败时

        Example:
            >>> connections = config_manager.list_connections()
            >>> print(connections)
            ["postgres_db", "mysql_db"]
        """
        try:
            config = self._load_config()
            return list(config["connections"].keys())
        except Exception as e:
            logger.error(f"列出连接失败: {str(e)}")
            raise ConfigError(f"连接列出失败: {str(e)}") from e

    def get_config_info(self) -> Dict[str, Any]:
        """
        获取配置文件的基本信息

        Returns:
            Dict[str, Any]: 配置文件信息字典

        Example:
            >>> info = config_manager.get_config_info()
            >>> print(info["version"])
            "1.0.0"
        """
        try:
            config = self._load_config()
            return {
                "version": config["version"],
                "app_name": config["app_name"],
                "connection_count": len(config["connections"]),
                "created": config["metadata"]["created"],
                "last_modified": config["metadata"]["last_modified"],
                "config_file": str(self.config_path),
            }
        except Exception as e:
            logger.error(f"获取配置信息失败: {str(e)}")
            raise ConfigError(f"配置信息获取失败: {str(e)}") from e

    def backup_config(self, backup_path: Path | None = None) -> Path:
        """
        备份配置文件

        Args:
            backup_path: 备份文件路径，如果为None则自动生成

        Returns:
            Path: 备份文件路径

        Raises:
            ConfigError: 当备份失败时

        Security Note:
            - 备份文件包含加密数据，应妥善保管
            - 建议定期备份重要配置

        Example:
            >>> backup_path = config_manager.backup_config()
            >>> print(backup_path)
            "path/to/connections.toml.backup.20231201_120000"
        """
        try:
            if backup_path is None:
                timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
                backup_path = self.config_dir / f"{self.config_file}.backup.{timestamp}"

            shutil.copy2(self.config_path, backup_path)
            logger.info(f"配置文件已备份: {backup_path}")
            return backup_path

        except Exception as e:
            logger.error(f"备份配置文件失败: {str(e)}")
            raise ConfigError(f"配置文件备份失败: {str(e)}") from e

    def __str__(self) -> str:
        """返回配置管理器的用户友好字符串表示"""
        try:
            config_info = self.get_config_info()
            connection_count = config_info["connection_count"]
            return f"ConfigManager('{self.app_name}', {connection_count} connections)"
        except Exception:
            # 如果获取配置信息失败，返回基本表示
            return f"ConfigManager('{self.app_name}', '{self.config_file}')"

    def __repr__(self) -> str:
        """返回配置管理器的详细表示，用于调试"""
        try:
            config_info = self.get_config_info()
            connection_count = config_info["connection_count"]
            version = config_info["version"]
            return (
                f"ConfigManager(app_name='{self.app_name}', "
                f"config_file='{self.config_file}', "
                f"config_path='{self.config_path}', "
                f"version='{version}', "
                f"connections={connection_count})"
            )
        except Exception:
            # 如果获取配置信息失败，返回基本表示
            return (
                f"ConfigManager(app_name='{self.app_name}', "
                f"config_file='{self.config_file}', "
                f"config_path='{self.config_path}')"
            )



import os
import re
import warnings
from abc import ABC
from datetime import datetime
from pathlib import Path
from typing import Any, Literal, Tuple

import jaydebeapi
from dateutil import parser
from sqlalchemy import CHAR, TIMESTAMP, VARCHAR, TypeDecorator, exc, sql, util
from sqlalchemy.dialects import registry
from sqlalchemy.dialects.oracle.base import OracleDialect
from sqlalchemy.engine.interfaces import DBAPIModule
from sqlalchemy.engine.url import make_url

from ..utils.path_utils import PathHelper


class GBase8sCursor(jaydebeapi.Cursor):
    """
    GBase 8s JDBC 游标类，用于处理 GBaseClob2 类型的特殊转换。

    继承自 jaydebeapi.Cursor，重写未知 SQL 类型转换器以正确处理 GBaseClob2 类型。
    """

    def __init__(self, connection: Any, converters: Any) -> None:
        """
        初始化 GBase 8s 游标。

        Args:
            connection: JDBC 连接对象
            converters: 类型转换器字典
        """
        super().__init__(connection, converters)
        jaydebeapi._unknownSqlTypeConverter = self._unknownSqlTypeConverter

    def _unknownSqlTypeConverter(self, result_set: Any, column_index: int) -> Any:
        """
        处理未知 SQL 类型的转换，特别处理 GBaseClob2 类型。

        Args:
            result_set: JDBC 结果集对象
            column_index: 列索引

        Returns:
            转换后的值，如果是 GBaseClob2 类型则转换为字符串
        """
        value = result_set.getObject(column_index)

        # 检查是否为 GBaseClob2 类型
        if str(type(value)) == "<java class 'com.gbasedbt.jdbc.GBaseClob2'>":
            string = ""
            reader = value.getCharacterStream()
            while True:
                char = reader.read()
                if char == -1:  # 到达流末尾
                    break
                string += chr(char)
            value = string

        return value


class ObTimestamp(TypeDecorator):
    """
    GBase 8s 时间戳类型装饰器。

    处理 Python datetime 对象与 GBase 8s 时间戳类型之间的转换。
    """

    impl = TIMESTAMP

    def process_bind_param(self, value: Any, dialect: Any) -> Any:
        """
        处理绑定参数，将 Python datetime 转换为 Java Timestamp。

        Args:
            value: 输入值
            dialect: SQLAlchemy 方言对象

        Returns:
            转换后的时间戳值
        """
        if isinstance(value, datetime):
            import jpype

            timestamp_class = jpype.JClass("java.sql.Timestamp")
            value = timestamp_class.valueOf(value.strftime("%Y-%m-%d %H:%M:%S.%f"))
        return value

    def process_result_value(self, value: Any, dialect: Any) -> datetime | None:
        """
        处理结果值，将字符串解析为 Python datetime 对象。

        Args:
            value: 数据库返回的值
            dialect: SQLAlchemy 方言对象

        Returns:
            解析后的 datetime 对象，如果值为 None 则返回 None
        """
        if value is not None:
            return parser.parse(value)
        return None


# 更新列规范，将 TIMESTAMP 类型替换为自定义的 ObTimestamp
colspecs = util.update_copy(
    dict(OracleDialect.colspecs),
    {
        TIMESTAMP: ObTimestamp,
        VARCHAR: VARCHAR,  # 确保VARCHAR类型正确映射
        CHAR: CHAR,  # CHAR类型映射
    },
)


class GBase8sJDBCDialect(OracleDialect, ABC):
    """
    GBase 8s JDBC 方言实现。

    基于 OracleDialect 实现，适配 GBase 8s 数据库的特性。
    """

    name = "gbasedbt-sqli"
    driver = "com.gbasedbt.jdbc.Driver"
    colspecs = colspecs

    # 数据库特性支持配置
    supports_native_decimal = True
    supports_sane_rowcount = False
    supports_sane_multi_rowcount = False
    supports_unicode_binds = True
    supports_statement_cache = True
    description_encoding = None

    def initialize(self, connection: Any) -> None:
        """
        初始化方言，避免调用 OracleDialect 的初始化方法。

        Args:
            connection: SQLAlchemy 连接对象
        """
        # 由于 GBase 8s 与 Oracle 有差异，不调用父类初始化
        pass

    @classmethod
    def dbapi(cls) -> Any:
        """
        返回数据库 API 模块，使用自定义的 GBase8sCursor。

        Returns:
            jaydebeapi 模块，使用自定义游标类
        """
        import jaydebeapi

        jaydebeapi.Cursor = GBase8sCursor
        return jaydebeapi

    @classmethod
    def import_dbapi(cls) -> DBAPIModule:
        """
        导入数据库 API 模块。

        Returns:
            导入的 jaydebeapi 模块
        """
        return __import__("jaydebeapi")

    def do_rollback(self, dbapi_connection: Any) -> None:
        """
        执行回滚操作。

        GBase 8s 不支持事务回滚操作，因此该方法为空实现。

        Args:
            dbapi_connection: 数据库 API 连接对象
        """
        # GBase 8s 不支持事务回滚，留空实现
        pass

    def create_connect_args(self, url: Any) -> Tuple[Tuple, dict]:
        """
        创建连接参数。

        Args:
            url: SQLAlchemy URL 对象

        Returns:
            包含连接参数的元组和字典
        """
        url_obj = make_url(url)

        # 构建 JDBC URL
        jdbc_url = self._build_jdbc_url(url_obj)

        # 构建连接参数
        connect_args = self._build_connect_args(url_obj)

        kwargs = {
            "jclassname": self.driver,
            "url": jdbc_url,
            "driver_args": connect_args,
        }

        # 处理 JAR 路径
        self._handle_jar_path(url_obj, kwargs)

        return (), kwargs

    def _build_jdbc_url(self, url_obj: Any) -> str:
        """构建 JDBC 连接 URL 字符串。"""
        jdbc_url_parts = [f"jdbc:{self.name}://{url_obj.host}"]
        if url_obj.port:
            jdbc_url_parts.append(f":{url_obj.port}")
        if url_obj.database:
            jdbc_url_parts.append(f"/{url_obj.database}")
        return "".join(jdbc_url_parts)

    def _build_connect_args(self, url_obj: Any) -> dict:
        """构建数据库连接参数字典。"""
        connect_args = {}
        if url_obj.username:
            connect_args["user"] = url_obj.username
        if url_obj.password:
            connect_args["password"] = url_obj.password

        if url_obj.query:
            for key, value in url_obj.query.items():
                connect_args[key] = value if isinstance(value, str) else str(value)

        if "rewriteBatchedStatements" not in connect_args:
            connect_args["rewriteBatchedStatements"] = "true"

        return connect_args

    def _handle_jar_path(self, url_obj: Any, kwargs: dict) -> None:
        """
        处理 JDBC 驱动 JAR 文件路径。

        优先级：URL参数 > 环境变量 > 默认路径

        Args:
            url_obj: SQLAlchemy URL对象
        """

        jar_path = None

        # 1. 优先使用URL参数
        if url_obj.query and "jarpath" in url_obj.query:
            jar_path = url_obj.query["jarpath"]

        # 2. 其次使用环境变量
        elif "GBASE8S_JDBC_JARPATH" in os.environ:
            jar_path = os.environ["GBASE8S_JDBC_JARPATH"]

        # 3. 尝试常见默认路径
        else:
            app_name = "db_connector_tool"
            config_dir = PathHelper.get_user_config_dir(app_name)
            config_path = config_dir / "jars"
            path = Path(config_path)
            # 使用 glob 递归获取所有文件
            files = [p for p in path.rglob("*gbase*.jar") if p.is_file()]
            for file in files:
                jar_path = str(file)
                break

        if isinstance(jar_path, str) and os.path.exists(jar_path):
            kwargs["jars"] = jar_path
        else:
            warnings.warn(f"JDBC 驱动 jar 文件不存在：{jar_path}", UserWarning)
            kwargs["jars"] = jar_path

    @property
    def _is_oracle_8(self) -> bool:
        """
        检查是否为 Oracle 8 数据库。

        Returns:
            False，因为这是 GBase 8s 方言
        """
        return False

    def _check_max_identifier_length(self, connection: Any) -> Literal[30] | None:
        """
        检查最大标识符长度。

        Args:
            connection: 数据库连接对象

        Returns:
            最大标识符长度，如果无法确定则返回 None
        """
        return None

    def get_default_schema_name(self, connection: Any) -> str | None:
        """
        获取默认 schema 名称。

        Args:
            connection: 数据库连接对象

        Returns:
            大写的用户名作为默认 schema，如果用户名不存在则返回 None
        """
        username = connection.engine.url.username
        return username.upper() if username else None

    def _get_default_schema_name(self, connection: Any) -> str | None:
        """
        内部方法：获取默认 schema 名称。

        Args:
            connection: 数据库连接对象

        Returns:
            默认 schema 名称
        """
        return self.get_default_schema_name(connection)

    def _get_server_version_info(self, connection: Any) -> Tuple[int, ...] | None:
        """
        获取服务器版本信息。

        Args:
            connection: 数据库连接对象

        Returns:
            版本号元组，如果获取失败则返回 None
        """
        try:
            # 尝试多种版本查询方式
            version_queries = [
                "select dbinfo('version_gbase','full') from dual",
                "select dbinfo('version','full') from dual",
            ]

            banner = None
            for query in version_queries:
                try:
                    version_sql = sql.text(query)
                    banner = connection.execute(version_sql).scalar()
                    if banner:
                        break
                except exc.DBAPIError:
                    continue

            if isinstance(banner, str):
                # 正则表达式匹配 GBase 8s 版本格式
                version_pattern = (
                    r"GBase8sV?([\d.]+)"  # 主版本号
                    r"(?:_TL_([\d.]+))?"  # TL 版本（可选）
                    r"(?:_(\d+X\d+))?"  # 特殊版本格式（可选）
                    r"(?:_(\d+))?"  # 构建号（可选）
                    r"(?:_([a-f0-9]+))?"  # 提交哈希（可选）
                )
                match = re.search(version_pattern, banner)

                if match:
                    version_str = match.group(1)
                    return tuple(int(part) for part in version_str.split("."))
                else:
                    # 尝试简单版本号提取
                    simple_match = re.search(r"(\d+)\.(\d+)", banner)
                    if simple_match:
                        return (int(simple_match.group(1)), int(simple_match.group(2)))

            return None

        except exc.DBAPIError:
            # 版本查询失败，返回 None
            return None
        except Exception:
            # 其他异常，返回 None
            return None

    def is_disconnect(self, e: Exception, connection: Any, cursor: Any) -> bool:
        """
        检查异常是否为连接断开错误。

        Args:
            e: 异常对象
            connection: 连接对象
            cursor: 游标对象

        Returns:
            如果是连接断开错误返回 True，否则返回 False
        """
        error_str = str(e).lower()
        disconnect_indicators = [
            "connection closed",
            "socket closed",
            "broken pipe",
            "connection reset",
            "jdbc connection",
            "network error",
        ]

        return any(indicator in error_str for indicator in disconnect_indicators)


# 注册方言到 SQLAlchemy
registry.register(
    "jdbcgbase8s", "db_connector_tool.drivers.gbase8s_jdbc", "GBase8sJDBCDialect"
)



"""
数据库连接管理器模块

提供统一的数据库连接管理接口， 专注于连接生命周期管理和统一API设计。
采用连接池技术实现连接复用， 提高性能和资源利用率。

主要特性：
- 连接池管理和连接复用
- 统一的错误处理和日志记录
- 线程安全的连接操作
- 连接生命周期管理
- 性能监控和统计信息
- 配置覆盖支持（临时连接配置）

设计原则：
- 简化配置验证，详细验证逻辑下放到SQLAlchemyDriver
- 增强连接池管理功能
- 明确职责边界，避免与底层组件功能重叠
- 提供简洁易用的用户接口
"""

import threading
import time
from typing import Any, Dict, List, Set

from ..drivers.sqlalchemy_driver import SQLAlchemyDriver
from ..utils.logging_utils import get_logger
from .config import ConfigManager
from .exceptions import ConfigError, ConnectionError, DatabaseError

# 获取模块级别的日志记录器
logger = get_logger(__name__)

# 错误消息常量
CONNECTION_NOT_FOUND_MSG = "连接配置不存在: {}"
CONNECTION_ALREADY_EXISTS_MSG = "连接配置已存在: {}"

# 支持的数据库类型
SUPPORTED_DATABASE_TYPES: Set[str] = {
    "oracle",
    "postgresql",
    "mysql",
    "mssql",
    "sqlite",
    "gbasedbt",
}

# 默认配置参数
DEFAULT_MAX_IDLE_TIME = 300  # 5分钟
DEFAULT_CONNECTION_TIMEOUT = 30  # 30秒连接超时
DEFAULT_POOL_RECYCLE = 3600  # 1小时连接回收


class ConnectionInfo:
    """
    连接信息封装类

    封装连接的统计信息和状态，用于连接池管理和监控。

    Attributes:
        driver (SQLAlchemyDriver): 数据库驱动实例
        last_used (float): 最后使用时间戳
        use_count (int): 使用次数统计
        created_at (float): 创建时间戳
        is_active (bool): 连接是否活跃状态

    Example:
        >>> connection_info = ConnectionInfo(driver)
        >>> connection_info.mark_used()
        >>> print(f"使用次数: {connection_info.use_count}")
    """

    def __init__(self, driver: SQLAlchemyDriver) -> None:
        """
        初始化连接信息

        Args:
            driver: 数据库驱动实例，必须已建立连接
        """
        self.driver = driver
        self.last_used = time.time()
        self.use_count = 0
        self.created_at = time.time()
        self.is_active = True

    def mark_used(self) -> None:
        """标记连接被使用，更新最后使用时间和使用次数"""
        self.last_used = time.time()
        self.use_count += 1

    def invalidate(self) -> None:
        """标记连接为无效状态"""
        self.is_active = False

    def __str__(self) -> str:
        """返回连接的字符串表示"""
        last_used_str = time.strftime(
            "%Y-%m-%d %H:%M:%S", time.localtime(self.last_used)
        )
        created_str = time.strftime(
            "%Y-%m-%d %H:%M:%S", time.localtime(self.created_at)
        )
        return (
            f"ConnectionInfo(use_count={self.use_count}, "
            f"last_used={last_used_str}, created_at={created_str}, "
            f"is_active={self.is_active})"
        )


class DatabaseManager:
    """
    数据库管理器类

    提供统一的数据库连接管理接口，实现连接池管理和生命周期控制。
    支持多种数据库类型，提供线程安全的连接操作。

    Attributes:
        app_name (str): 应用名称，用于配置文件的命名空间
        config_file (str): 配置文件名
        config_manager (ConfigManager): 配置管理器实例
        connection_pool (Dict[str, ConnectionInfo]): 连接池字典
        _lock (threading.RLock): 可重入锁，确保线程安全
        _statistics (Dict[str, Any]): 连接统计信息

    Example:
        >>> db_manager = DatabaseManager("my_application")
        >>> config = {
        ...     "type": "mysql",
        ...     "host": "localhost",
        ...     "port": 3306,
        ...     "username": "user",
        ...     "password": "pass",
        ...     "database": "test_db"
        ... }
        >>> db_manager.add_connection("mysql_db", config)
        >>> results = db_manager.execute_query("mysql_db", "SELECT * FROM users")
    """

    def __init__(
        self, app_name: str = "db_connector_tool", config_file: str = "connections.toml"
    ) -> None:
        """
        初始化数据库管理器

        Args:
            app_name: 应用名称，用于配置文件的命名空间和日志标识
            config_file: 配置文件名，默认为"connections.toml"

        Raises:
            ConfigError: 当配置管理器初始化失败时

        Note:
            建议为每个应用使用唯一的 app_name，避免配置冲突和日志混淆

        Example:
            >>> db_manager = DatabaseManager("my_app", "database.toml")
            >>> print(f"应用名称: {db_manager.app_name}")
        """
        try:
            self.app_name = app_name
            self.config_file = config_file
            self.config_manager = ConfigManager(app_name, config_file)
            self.connection_pool: Dict[str, ConnectionInfo] = {}
            self._lock = threading.RLock()
            self._statistics = {
                "total_connections_created": 0,
                "total_connections_closed": 0,
                "connection_errors": 0,
                "idle_connections_cleaned": 0,
                "start_time": time.time(),
                "last_cleanup_time": time.time(),
            }
            logger.info(f"数据库管理器初始化成功: {app_name}")
        except Exception as e:
            logger.error(f"初始化数据库管理器失败: {str(e)}")
            raise ConfigError(f"数据库管理器初始化失败: {str(e)}")

    def add_connection(self, name: str, connection_config: Dict[str, Any]) -> None:
        """
        创建数据库连接配置

        Args:
            name: 连接名称，用于标识不同的数据库连接
            connection_config: 连接配置字典

        Raises:
            DatabaseError: 当创建连接配置失败时
            ConfigError: 当连接配置验证失败或连接已存在时

        Process:
            1. 检查连接名称是否已存在
            2. 验证基本配置
            3. 保存到配置管理器
            4. 记录日志

        Example:
            >>> config = {
            ...     "type": "mysql",
            ...     "host": "localhost",
            ...     "port": 3306,
            ...     "username": "root",
            ...     "password": "password",
            ...     "database": "test_db"
            ... }
            >>> db_manager.add_connection("mysql_db", config)
        """
        with self._lock:
            try:
                # 检查连接是否已存在
                existing_connections = self.list_connections()
                if name in existing_connections:
                    raise ConfigError(CONNECTION_ALREADY_EXISTS_MSG.format(name))

                # 简化配置验证
                self._validate_basic_config(connection_config)

                # 保存到配置管理器
                self.config_manager.add_connection(name, connection_config)
                logger.info(f"数据库连接配置已创建: {name}")

            except ConfigError:
                raise
            except Exception as e:
                logger.error(f"创建数据库连接配置失败 {name}: {str(e)}")
                raise DatabaseError(f"数据库连接配置创建失败: {str(e)}")

    def _validate_basic_config(self, config: Dict[str, Any]) -> None:
        """
        基本配置验证

        Args:
            config: 连接配置字典

        Raises:
            ConfigError: 当基本配置验证失败时

        Note:
            详细验证逻辑由SQLAlchemyDriver处理，这里只做基本检查
            确保配置结构正确，避免无效配置进入系统

        Validation Rules:
            - 配置不能为空且必须是字典
            - 数据库类型必须在支持列表中
            - 非SQLite数据库需要基本连接参数
        """
        if not config or not isinstance(config, dict):
            raise ConfigError("连接配置不能为空且必须是字典")

        # 验证数据库类型
        db_type = config.get("type", "").lower()
        if db_type not in SUPPORTED_DATABASE_TYPES:
            supported_types = ", ".join(sorted(SUPPORTED_DATABASE_TYPES))
            raise ConfigError(
                f"不支持的数据库类型: {db_type}，支持的类型: {supported_types}"
            )

        # SQLite数据库特殊处理
        if db_type == "sqlite" and "database" not in config:
            config["database"] = ":memory:"
            return

        # 应用特定数据库类型的默认配置
        self._apply_default_config(config)

        # 验证必需参数
        required_fields = ["username", "password", "host"]
        missing_fields = [field for field in required_fields if field not in config]
        if missing_fields:
            raise ConfigError(f"缺少必需的连接参数: {', '.join(missing_fields)}")

    def _apply_default_config(self, config: Dict[str, Any]) -> None:
        """
        应用特定数据库类型的默认配置

        Args:
            config: 连接配置字典
        """
        db_type = config.get("type", "").lower()

        if db_type == "oracle" and "service_name" not in config:
            config["service_name"] = "XE"
        elif db_type == "postgresql" and "gssencmode" not in config:
            config["gssencmode"] = "disable"
        elif db_type == "mssql":
            if "charset" not in config:
                config["charset"] = "cp936"
            if "tds_version" not in config:
                config["tds_version"] = "7.0"
        elif db_type == "gbasedbt" and "server" not in config:
            config["server"] = "gbase01"

    def remove_connection(self, name: str) -> None:
        """
        删除连接配置

        Args:
            name: 连接名称

        Raises:
            DatabaseError: 当删除连接配置失败时
            ConfigError: 当连接配置不存在时

        Process:
            1. 验证连接存在
            2. 清理连接池中的连接
            3. 删除配置信息
            4. 记录日志

        Note:
            删除前会自动关闭对应的连接并清理缓存
            此操作不可逆，建议谨慎使用

        Example:
            >>> db_manager.remove_connection("mysql_db")
        """
        with self._lock:
            try:
                # 检查连接是否存在
                self._validate_connection_exists(name)

                # 先关闭连接
                self._cleanup_connection(name)

                # 删除配置
                self.config_manager.remove_connection(name)
                logger.info(f"连接配置已删除: {name}")

            except ConfigError:
                raise
            except Exception as e:
                logger.error(f"删除连接配置失败 {name}: {str(e)}")
                raise DatabaseError(f"连接配置删除失败: {str(e)}")

    def _validate_connection_exists(self, name: str) -> None:
        """
        验证连接配置是否存在

        Args:
            name: 连接名称

        Raises:
            ConfigError: 当连接配置不存在时

        Note:
            此方法用于内部验证，确保操作前连接配置存在
        """
        existing_connections = self.list_connections()
        if name not in existing_connections:
            raise ConfigError(CONNECTION_NOT_FOUND_MSG.format(name))

    def _cleanup_connection(self, name: str) -> None:
        """
        安全清理连接资源

        Args:
            name: 连接名称

        Note:
            安全地关闭连接并清理相关资源，确保无资源泄漏
            即使发生异常也不会影响其他连接，但会记录详细错误
        """
        if name not in self.connection_pool:
            logger.debug(f"连接 {name} 不在连接池中，无需清理")
            return

        connection_info = self.connection_pool[name]

        try:
            # 标记连接为无效状态
            connection_info.invalidate()

            # 安全关闭连接
            if (
                hasattr(connection_info.driver, "is_connected")
                and connection_info.driver.is_connected
            ):
                try:
                    connection_info.driver.disconnect()
                    logger.debug(f"连接 {name} 已安全关闭")
                except Exception as disconnect_error:
                    logger.warning(
                        f"关闭连接 {name} 时发生异常: {str(disconnect_error)}"
                    )
            else:
                logger.debug(f"连接 {name} 未连接或已关闭")

        except Exception as e:
            logger.error(f"清理连接 {name} 时发生严重异常: {str(e)}")
        finally:
            # 确保从连接池中移除，避免内存泄漏
            try:
                del self.connection_pool[name]
                self._statistics["total_connections_closed"] += 1
                logger.debug(f"连接 {name} 已从连接池中移除")
            except KeyError:
                logger.debug(f"连接 {name} 已从连接池中移除")
            except Exception as e:
                logger.error(f"从连接池中移除连接 {name} 时发生异常: {str(e)}")

    def update_connection(self, name: str, connection_config: Dict[str, Any]) -> None:
        """
        更新连接配置

        Args:
            name: 连接名称
            connection_config: 新的连接配置

        Raises:
            DatabaseError: 当更新连接配置失败时
            ConfigError: 当新的连接配置验证失败或连接不存在时

        Process:
            1. 验证连接存在和新配置
            2. 关闭现有连接
            3. 更新配置信息
            4. 记录日志

        Note:
            更新配置前会关闭现有连接，新连接将在下次获取时创建
            确保配置更新不会影响正在进行的操作

        Example:
            >>> new_config = {
            ...     "type": "mysql",
            ...     "host": "new_host",
            ...     "port": 3306,
            ...     "username": "root",
            ...     "password": "new_password",
            ...     "database": "test_db"
            ... }
            >>> db_manager.update_connection("mysql_db", new_config)
        """
        with self._lock:
            try:
                # 检查连接是否存在
                self._validate_connection_exists(name)

                # 简化配置验证
                self._validate_basic_config(connection_config)

                # 关闭现有连接
                self._cleanup_connection(name)

                # 更新配置
                self.config_manager.update_connection(name, connection_config)
                logger.info(f"连接配置已更新: {name}")

            except ConfigError:
                raise
            except Exception as e:
                logger.error(f"更新连接配置失败 {name}: {str(e)}")
                raise DatabaseError(f"连接配置更新失败: {str(e)}")

    def show_connection(self, name: str) -> Dict[str, Any]:
        """
        显示指定连接的配置信息

        Args:
            name: 连接名称

        Returns:
            Dict[str, Any]: 连接配置信息字典

        Raises:
            ConfigError: 当连接配置不存在时

        Example:
            >>> config = db_manager.show_connection("mysql_db")
            >>> print(f"主机: {config['host']}")
        """
        return self.config_manager.get_connection(name)

    def list_connections(self) -> List[str]:
        """
        获取所有可用的连接名称

        Returns:
            连接名称列表

        Example:
            >>> connections = db_manager.list_connections()
            >>> print(f"可用的连接: {', '.join(connections)}")
        """
        return self.config_manager.list_connections()

    def get_connection(
        self, name: str, config_overrides: Dict[str, Any] | None = None
    ) -> SQLAlchemyDriver:
        """
        获取数据库连接（连接池管理）

        Args:
            name: 连接名称
            config_overrides: 可选的配置覆盖字典，用于临时修改连接配置
                            例如：{"host": "127.0.0.1", "port": 3306}

        Returns:
            SQLAlchemyDriver实例，用于执行数据库操作

        Raises:
            DatabaseError: 当获取连接失败时
            ConnectionError: 当连接建立失败时
            ConfigError: 当连接配置不存在时

        Process:
            1. 验证连接配置存在
            2. 检查连接池中是否有有效连接
            3. 如果连接无效，清理并重新创建
            4. 返回可用的驱动实例

        Note:
            如果连接已存在且有效，直接返回缓存的连接，否则创建新连接
            连接池机制提高性能，避免频繁创建和销毁连接
            当提供config_overrides时，会创建新的连接而不是使用缓存

        Example:
            >>> driver = db_manager.get_connection("mysql_db")
            >>> result = driver.execute_query("SELECT * FROM users")
            >>> # 使用配置覆盖
            >>> driver = db_manager.get_connection("mysql_db", {"host": "127.0.0.1"})
        """
        with self._lock:
            try:
                # 检查连接配置是否存在
                self._validate_connection_exists(name)

                # 如果提供了配置覆盖，则强制创建新连接
                if config_overrides:
                    # 清理可能存在的缓存连接
                    if name in self.connection_pool:
                        self._cleanup_connection(name)

                    # 获取基础配置并应用覆盖
                    base_config = self.show_connection(name)
                    connection_config = {**base_config, **config_overrides}

                    # 验证修改后的配置
                    self._validate_basic_config(connection_config)

                    # 创建新连接
                    driver = SQLAlchemyDriver(connection_config)
                    driver.connect()

                    # 创建连接信息但不加入连接池（因为是临时配置）
                    connection_info = ConnectionInfo(driver)
                    # 注意：临时配置的连接不加入连接池，避免配置冲突

                    logger.info(f"使用临时配置建立数据库连接: {name}")
                    return driver

                # 连接池管理逻辑（无配置覆盖时）
                if name in self.connection_pool:
                    connection_info = self.connection_pool[name]

                    # 检查连接是否有效
                    if connection_info.is_active and self._is_connection_valid(
                        connection_info.driver
                    ):
                        connection_info.mark_used()
                        logger.debug(f"使用缓存的数据库连接: {name}")
                        return connection_info.driver
                    else:
                        # 连接无效，清理并重新创建
                        self._cleanup_connection(name)

                # 创建新连接
                connection_config = self.show_connection(name)
                driver = SQLAlchemyDriver(connection_config)
                driver.connect()

                # 创建连接信息并加入连接池
                connection_info = ConnectionInfo(driver)
                self.connection_pool[name] = connection_info
                self._statistics["total_connections_created"] += 1

                logger.info(f"数据库连接已建立: {name}")
                return driver

            except (ConfigError, ConnectionError):
                raise
            except Exception as e:
                self._statistics["connection_errors"] += 1
                logger.error(f"获取数据库连接失败 {name}: {str(e)}")
                raise DatabaseError(f"数据库连接获取失败: {str(e)}")

    def _is_connection_valid(self, driver: SQLAlchemyDriver) -> bool:
        """
        全面的连接有效性检查

        Args:
            driver: 数据库驱动实例

        Returns:
            bool: 连接是否有效

        Note:
            提供多层次的连接有效性验证，包括基本状态检查和实际查询测试
        """
        try:
            # 第一层检查：基本属性验证
            if not hasattr(driver, "is_connected"):
                logger.debug("驱动实例缺少is_connected属性")
                return False

            if not driver.is_connected:
                logger.debug("驱动实例标记为未连接状态")
                return False

            # 第二层检查：实际查询测试
            return driver.test_connection()

        except Exception as e:
            logger.debug(f"连接有效性检查失败: {str(e)}")
            return False

    def test_connection(self, name: str) -> bool:
        """
        测试连接是否有效

        Args:
            name: 连接名称

        Returns:
            True表示连接成功，False表示连接失败

        Note:
            测试失败时会记录详细的错误信息，但不会抛出异常
            适合用于健康检查或连接监控

        Example:
            >>> if db_manager.test_connection("mysql_db"):
            ...     print("连接测试成功")
            ... else:
            ...     print("连接测试失败")
        """
        try:
            driver = self.get_connection(name)
            success = driver.test_connection()
            if success:
                logger.debug(f"连接测试成功: {name}")
            else:
                logger.warning(f"连接测试失败: {name}")
            return success
        except Exception as e:
            logger.error(f"连接测试失败 {name}: {str(e)}")
            return False

    def execute_query(
        self, connection_name: str, query: str, params: Dict[str, Any] | None = None
    ) -> List[Dict[str, Any]]:
        """
        执行SQL查询语句

        Args:
            connection_name: 连接名称
            query: SQL查询语句
            params: 查询参数字典，可选

        Returns:
            查询结果列表，每行数据为字典格式

        Raises:
            DatabaseError: 当查询执行失败时
            ConfigError: 当连接配置不存在时

        Process:
            1. 获取数据库连接
            2. 执行查询语句
            3. 返回格式化结果

        Note:
            此方法封装了连接获取和错误处理，提供统一的查询接口
            适合执行SELECT等查询操作

        Example:
            >>> results = db_manager.execute_query(
            ...     "mysql_db",
            ...     "SELECT * FROM users WHERE age > :age",
            ...     {"age": 18}
            ... )
        """
        try:
            driver = self.get_connection(connection_name)
            with driver:
                return driver.execute_query(query, params)
        except (ConfigError, ConnectionError):
            raise
        except Exception as e:
            logger.error(f"执行查询失败: {str(e)}")
            raise DatabaseError(f"查询执行失败: {str(e)}")

    def execute_command(
        self,
        connection_name: str,
        command: str,
        params: Dict[str, Any] | None = None,
    ) -> int:
        """
        执行非查询SQL命令（INSERT/UPDATE/DELETE等）

        Args:
            connection_name: 连接名称
            command: SQL命令语句
            params: 命令参数字典，可选

        Returns:
            影响的行数

        Raises:
            DatabaseError: 当命令执行失败时
            ConfigError: 当连接配置不存在时

        Process:
            1. 获取数据库连接
            2. 执行命令语句
            3. 返回影响行数

        Note:
            适合执行数据修改操作，返回受影响的行数
            提供事务安全的命令执行环境

        Example:
            >>> affected_rows = db_manager.execute_command(
            ...     "mysql_db",
            ...     "UPDATE users SET name = :name WHERE id = :id",
            ...     {"name": "Bob", "id": 1}
            ... )
        """
        try:
            driver = self.get_connection(connection_name)
            with driver:
                return driver.execute_command(command, params)
        except (ConfigError, ConnectionError):
            raise
        except Exception as e:
            logger.error(f"执行命令失败: {str(e)}")
            raise DatabaseError(f"命令执行失败: {str(e)}")

    def close_connection(self, name: str) -> None:
        """
        关闭数据库连接

        Args:
            name: 连接名称

        Raises:
            DatabaseError: 当关闭连接失败时

        Note:
            关闭连接会释放相关资源，但保留连接配置
            连接可以在需要时重新建立

        Example:
            >>> db_manager.close_connection("mysql_db")
        """
        with self._lock:
            try:
                self._cleanup_connection(name)
                logger.info(f"数据库连接已关闭: {name}")
            except Exception as e:
                logger.error(f"关闭连接失败 {name}: {str(e)}")
                raise DatabaseError(f"连接关闭失败: {str(e)}")

    def close_all_connections(self) -> None:
        """
        安全关闭所有数据库连接

        彻底清理所有活跃的连接资源，确保无资源泄漏。

        Raises:
            DatabaseError: 当关闭所有连接失败时

        Process:
            1. 获取所有连接名称的副本
            2. 逐个安全关闭连接
            3. 统计成功和失败数量
            4. 记录详细汇总日志
            5. 清理连接池字典

        Note:
            使用list()创建副本确保线程安全，避免在迭代过程中修改字典
            适合在应用关闭或维护时调用

        Example:
            >>> db_manager.close_all_connections()
        """
        with self._lock:
            try:
                # 创建连接名称副本，避免迭代过程中字典修改
                connection_names = list(self.connection_pool.keys())
                total_connections = len(connection_names)
                success_count = 0
                error_count = 0

                if total_connections == 0:
                    logger.info("连接池为空，无需关闭连接")
                    return

                logger.info(f"开始关闭所有连接，共 {total_connections} 个连接")

                for name in connection_names:
                    try:
                        # 使用内部清理方法，避免递归调用close_connection
                        self._cleanup_connection(name)
                        success_count += 1
                        logger.debug(f"连接 {name} 关闭成功")
                    except Exception as e:
                        error_count += 1
                        logger.error(f"关闭连接 {name} 失败: {str(e)}")

                # 最终检查连接池是否完全清空
                remaining_connections = len(self.connection_pool)
                if remaining_connections > 0:
                    logger.warning(
                        f"连接池清理不完整，仍有 {remaining_connections} 个连接未清理"
                    )
                    # 强制清空连接池
                    self.connection_pool.clear()
                    logger.info("已强制清空连接池")

                # 记录详细汇总信息
                if error_count > 0:
                    logger.warning(
                        f"关闭所有连接完成，成功: {success_count}, 失败: {error_count}, 总数: {total_connections}"
                    )
                else:
                    logger.info(f"所有数据库连接已安全关闭，共 {success_count} 个连接")

            except Exception as e:
                logger.error(f"关闭所有连接时发生严重异常: {str(e)}")
                # 即使发生异常也要尝试清理连接池
                try:
                    self.connection_pool.clear()
                    logger.info("异常情况下已强制清空连接池")
                except Exception:
                    pass
                raise DatabaseError(f"关闭所有连接失败: {str(e)}")

    def get_connection_info(self, name: str) -> Dict[str, Any]:
        """
        获取连接详细信息（包含统计信息）

        Args:
            name: 连接名称

        Returns:
            连接信息字典，包含基本配置和统计信息

        Raises:
            DatabaseError: 当获取连接信息失败时
            ConfigError: 当连接配置不存在时

        Example:
            >>> info = db_manager.get_connection_info("mysql_db")
            >>> print(f"使用次数: {info['use_count']}")
            >>> print(f"最后使用时间: {info['last_used']}")
        """
        with self._lock:
            try:
                # 检查连接是否存在
                self._validate_connection_exists(name)

                # 获取基本配置信息
                config = self.show_connection(name)
                info: Dict[str, Any] = {
                    "type": config.get("type"),
                    "host": config.get("host"),
                    "port": config.get("port"),
                    "database": config.get("database"),
                }

                # 添加连接池统计信息
                if name in self.connection_pool:
                    connection_info = self.connection_pool[name]
                    info.update(
                        {
                            "use_count": connection_info.use_count,
                            "last_used": connection_info.last_used,
                            "created_at": connection_info.created_at,
                            "is_active": connection_info.is_active,
                        }
                    )

                return {k: v for k, v in info.items() if v is not None}

            except ConfigError:
                raise
            except Exception as e:
                logger.error(f"获取连接信息失败 {name}: {str(e)}")
                raise DatabaseError(f"连接信息获取失败: {str(e)}")

    def get_statistics(self) -> Dict[str, Any]:
        """
        获取连接统计信息

        Returns:
            统计信息字典，包含连接创建、关闭、错误等统计

        Statistics Include:
            - 总连接创建次数
            - 总连接关闭次数
            - 连接错误次数
            - 系统运行时间
            - 当前活跃连接数
            - 连接池大小

        Example:
            >>> stats = db_manager.get_statistics()
            >>> print(f"总连接数: {stats['total_connections_created']}")
            >>> print(f"运行时间: {stats['uptime']}秒")
        """
        with self._lock:
            stats = self._statistics.copy()
            stats["current_time"] = time.time()
            stats["uptime"] = stats["current_time"] - stats["start_time"]
            stats["active_connections"] = len(
                [conn for conn in self.connection_pool.values() if conn.is_active]
            )
            stats["connection_pool_size"] = len(self.connection_pool)
            return stats

    def cleanup_idle_connections(
        self, max_idle_time: int = DEFAULT_MAX_IDLE_TIME
    ) -> int:
        """
        清理空闲时间过长的连接

        Args:
            max_idle_time: 最大空闲时间（秒），默认5分钟

        Returns:
            int: 清理的连接数量

        Raises:
            DatabaseError: 当清理过程发生严重错误时

        Process:
            1. 检查所有连接的空闲时间
            2. 清理超过最大空闲时间的连接
            3. 记录清理统计信息

        Example:
            >>> cleaned_count = db_manager.cleanup_idle_connections(600)  # 10分钟
            >>> print(f"清理了 {cleaned_count} 个空闲连接")
        """
        with self._lock:
            try:
                current_time = time.time()
                connection_names = list(self.connection_pool.keys())
                cleaned_count = 0

                if not connection_names:
                    logger.debug("连接池为空，无需清理空闲连接")
                    return 0

                logger.info(f"开始清理空闲连接，最大空闲时间: {max_idle_time}秒")

                for name in connection_names:
                    if name not in self.connection_pool:
                        continue

                    connection_info = self.connection_pool[name]
                    idle_time = current_time - connection_info.last_used

                    if idle_time > max_idle_time:
                        try:
                            logger.debug(
                                f"连接 {name} 空闲时间 {idle_time:.1f}秒超过限制，执行清理"
                            )
                            self._cleanup_connection(name)
                            cleaned_count += 1
                            self._statistics["idle_connections_cleaned"] += 1
                        except Exception as e:
                            logger.warning(f"清理空闲连接 {name} 失败: {str(e)}")

                self._statistics["last_cleanup_time"] = current_time

                if cleaned_count > 0:
                    logger.info(f"空闲连接清理完成，共清理 {cleaned_count} 个连接")
                else:
                    logger.debug("未发现需要清理的空闲连接")

                return cleaned_count

            except Exception as e:
                logger.error(f"清理空闲连接时发生异常: {str(e)}")
                raise DatabaseError(f"空闲连接清理失败: {str(e)}")

    def get_connection_pool_status(self) -> Dict[str, Any]:
        """
        获取连接池状态信息

        Returns:
            Dict[str, Any]: 连接池状态信息字典

        Example:
            >>> status = db_manager.get_connection_pool_status()
            >>> print(f"活跃连接数: {status['active_connections']}")
            >>> print(f"总创建连接数: {status['total_created']}")
        """
        with self._lock:
            current_time = time.time()

            # 计算连接池统计信息
            active_connections = 0
            total_use_count = 0
            max_idle_time = 0

            for name, connection_info in self.connection_pool.items():
                if connection_info.is_active:
                    active_connections += 1
                    total_use_count += connection_info.use_count
                    idle_time = current_time - connection_info.last_used
                    max_idle_time = max(max_idle_time, idle_time)

            return {
                "active_connections": active_connections,
                "total_connections": len(self.connection_pool),
                "total_use_count": total_use_count,
                "max_idle_time": max_idle_time,
                "statistics": self._statistics.copy(),
                "uptime": current_time - self._statistics["start_time"],
                "last_cleanup": current_time - self._statistics["last_cleanup_time"],
            }

    def __str__(self) -> str:
        """返回数据库管理器的字符串表示"""
        stats = self.get_statistics()
        return (
            f"DatabaseManager(app_name='{self.app_name}', "
            f"active_connections={stats['active_connections']}, "
            f"pool_size={stats['connection_pool_size']})"
        )



"""
SQLAlchemy 数据库驱动模块

提供基于 SQLAlchemy 的统一数据库连接接口，支持多种数据库类型：
- Oracle
- PostgreSQL
- MySQL
- SQL Server
- SQLite
- GBase 8s

该模块封装了数据库连接管理、查询执行、连接池配置和错误处理等功能，
提供线程安全的数据库操作接口。

主要特性：
- 多数据库类型支持
- 连接池管理和优化
- 线程安全操作
- 上下文管理器支持
- 详细的错误处理和日志记录
- 自动重连机制
- 连接有效性验证
"""

import re
from typing import Any, Dict, List, Optional, Tuple
from urllib.parse import quote_plus

from sqlalchemy import create_engine, text
from sqlalchemy.dialects.postgresql.base import PGDialect
from sqlalchemy.engine import Engine, Result
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import scoped_session, sessionmaker

from ..core.exceptions import ConnectionError, DriverError, QueryError
from ..utils.logging_utils import get_logger

# 获取模块级别的日志记录器
logger = get_logger(__name__)


def parse_kingbase_version(self, connection) -> Tuple[int, ...]:
    """
    解析 Kingbase 数据库版本信息

    Args:
        connection: 数据库连接对象

    Returns:
        Tuple[int, ...]: 版本号元组，如 (8, 6, 0)

    Raises:
        AssertionError: 当无法从版本字符串中解析出版本信息时

    Example:
        >>> version = parse_kingbase_version(connection)
        >>> print(version)
        (8, 6, 0)
    """
    v = connection.exec_driver_sql("select pg_catalog.version()").scalar()
    m = re.match(
        r".*(?:PostgreSQL|EnterpriseDB) "
        r"(\d+)\.?(\d+)?(?:\.(\d+))?(?:\.\d+)?(?:devel|beta)?",
        v,
    ) or re.search(r"V(\d+)R(\d+)C(\d+)B(\d+)", v)

    if not m:
        raise AssertionError(f"无法从字符串 '{v}' 中解析版本信息")

    return tuple(int(x) for x in m.group(1, 2, 3) if x is not None)


# 为 PostgreSQL 方言设置版本解析方法
PGDialect._get_server_version_info = parse_kingbase_version


class SQLAlchemyDriver:
    """
    SQLAlchemy 数据库驱动类

    提供统一的数据库连接和操作接口，支持多种数据库类型。
    使用连接池管理数据库连接，提高性能和资源利用率。

    主要特性：
    - 多数据库类型支持（Oracle、PostgreSQL、MySQL、SQL Server、SQLite、GBase 8s）
    - 连接池管理和优化配置
    - 线程安全操作，支持并发访问
    - 上下文管理器支持，自动资源管理
    - 详细的错误处理和日志记录
    - 连接有效性验证和自动重连
    - 参数化查询支持，防止SQL注入

    Attributes:
        URL_TEMPLATES (Dict[str, str]): 各数据库类型的连接URL模板
        TEST_QUERY_DEFAULT (str): 默认测试查询语句
        ORACLE_TEST_QUERY (str): Oracle专用测试查询语句

    Example:
        >>> config = {
        ...     "type": "mysql",
        ...     "username": "user",
        ...     "password": "pass",
        ...     "host": "localhost",
        ...     "database": "test_db",
        ...     "pool_config": {"pool_size": 5}
        ... }
        >>> driver = SQLAlchemyDriver(config)
        >>> with driver:
        ...     results = driver.execute_query("SELECT * FROM users")
        ...     print(results)
    """

    # 连接URL模板配置
    URL_TEMPLATES: Dict[str, str] = {
        "oracle": "oracle+oracledb://{username}:{password}@{host}:{port}/?service_name={service_name}",
        "postgresql": "postgresql+psycopg://{username}:{password}@{host}:{port}/{database}?gssencmode={gssencmode}",
        "mysql": "mysql+pymysql://{username}:{password}@{host}:{port}/{database}",
        "mssql": "mssql+pymssql://{username}:{password}@{host}:{port}/{database}?charset={charset}&tds_version={tds_version}",
        "sqlite": "sqlite:///{database}",
        "gbasedbt": "jdbcgbase8s://{host}:{port}/{database}:GBASEDBTSERVER={server}?user={username}&password={password}",
    }

    # 测试查询常量定义
    TEST_QUERY_DEFAULT: str = "SELECT 1"
    ORACLE_TEST_QUERY: str = "SELECT 1 FROM DUAL"

    def __init__(self, connection_config: Dict[str, Any]) -> None:
        """
        初始化数据库驱动实例

        Args:
            connection_config: 数据库连接配置字典，必需包含以下字段：
                - type (str): 数据库类型 (oracle/postgresql/mysql/mssql/sqlite/gbasedbt)
                - username (str): 用户名
                - password (str): 密码
                - host (str): 主机地址
                - port (int, optional): 端口号，如未提供将使用默认端口
                - database (str): 数据库名
                - pool_config (Dict[str, Any], optional): 连接池配置
                - echo_sql (bool, optional): 是否输出SQL日志，默认False

        Raises:
            ValueError: 当连接配置为空、非字典类型或缺少必需参数时
            DriverError: 当数据库类型不支持时

        Notes:
            - 配置验证在初始化时进行，确保配置有效性
            - 支持配置参数的灵活性和扩展性
            - 自动验证数据库类型支持

        Example:
            >>> config = {
            ...     "type": "mysql",
            ...     "username": "user",
            ...     "password": "pass",
            ...     "host": "localhost",
            ...     "database": "test_db",
            ...     "pool_config": {"pool_size": 5}
            ... }
            >>> driver = SQLAlchemyDriver(config)
        """
        if not connection_config:
            raise ValueError("连接配置不能为空")

        if not isinstance(connection_config, dict):
            raise ValueError("连接配置必须为字典类型")

        self.connection_config = connection_config
        self.engine: Optional[Engine] = None
        self.session_factory: Optional[scoped_session] = None
        self._connected: bool = False

        # 验证数据库类型支持
        db_type = self.connection_config.get("type", "").lower()
        if db_type not in self.URL_TEMPLATES:
            supported_types = ", ".join(self.URL_TEMPLATES.keys())
            raise DriverError(
                f"不支持的数据库类型: {db_type}，支持的类型: {supported_types}"
            )

        logger.debug(f"SQLAlchemy驱动实例初始化成功，数据库类型: {db_type}")

    def connect(self) -> None:
        """
        建立数据库连接并验证连接有效性

        该方法执行完整的数据库连接流程，包括：
        1. 检查当前连接状态，避免重复连接
        2. 构建数据库连接URL
        3. 配置连接池参数和引擎选项
        4. 创建SQLAlchemy引擎和会话工厂
        5. 执行实际连接测试验证连接有效性
        6. 更新连接状态并记录日志

        Raises:
            ConnectionError: 当连接建立或验证失败时抛出

        Notes:
            - 支持连接池配置，包括连接预检查、回收时间等
            - 自动适配不同数据库类型的特定配置
            - 严格区分引擎创建成功和实际连接成功
            - 连接失败时会自动清理已创建的引擎资源

        Example:
            >>> driver = SQLAlchemyDriver(connection_config)
            >>> driver.connect()  # 建立数据库连接
            >>> print(driver.is_connected)  # 检查连接状态
            True
        """
        try:
            # 检查连接状态，避免重复连接
            if self._connected:
                logger.warning("数据库连接已存在，无需重复连接")
                return

            # 构建数据库连接URL
            connection_url = self._build_connection_url()

            # 配置连接池参数
            pool_config = self._get_pool_config()

            # 准备引擎配置参数
            engine_kwargs: Dict[str, Any] = {
                "echo": self.connection_config.get("echo_sql", False),
                **pool_config,
            }

            # 添加数据库特定的引擎配置
            db_type = self.connection_config.get("type", "").lower()
            db_specific_config = self._get_engine_config(db_type)
            engine_kwargs.update(db_specific_config)

            # 创建SQLAlchemy引擎和会话工厂
            self.engine = create_engine(connection_url, **engine_kwargs)
            self.session_factory = scoped_session(sessionmaker(bind=self.engine))

            logger.info(
                f"数据库引擎创建成功: {self.connection_config.get('type')} "
                f"({self.connection_config.get('host') or self.connection_config.get('database')})"
            )

            # 执行实际连接测试验证连接有效性
            self._validate_connection()

        except SQLAlchemyError as e:
            # 连接过程中发生异常，清理资源并抛出错误
            self._cleanup_on_connection_failure()
            error_msg = f"数据库连接建立失败: {e.__class__.__name__}: {str(e)}"
            logger.error(error_msg, exc_info=True)
            raise ConnectionError(error_msg)

    def _build_connection_url(self) -> str:
        """
        构建数据库连接URL

        根据配置参数构建完整的数据库连接URL，包含必要的编码和参数处理。

        Returns:
            str: 完整的数据库连接URL字符串

        Raises:
            DriverError: 当缺少必需参数或数据库类型不支持时

        Notes:
            - 对用户名、密码和主机名中的特殊字符进行URL编码
            - 自动设置默认端口
            - 支持数据库特定的查询参数
            - SQLite数据库特殊处理，支持内存数据库
            - Oracle数据库强制使用Easy Connect格式，避免TNS别名查找

        Example:
            >>> url = driver._build_connection_url()
            >>> print(url)
            mysql+pymysql://user:pass@localhost:3306/test_db
        """
        db_type = self.connection_config.get("type", "").lower()
        config = self.connection_config.copy()

        # SQLite数据库特殊处理
        if db_type == "sqlite":
            if "database" not in config:
                config["database"] = ":memory:"  # 内存数据库
            return self.URL_TEMPLATES[db_type].format(**config)

        # 验证必需参数
        required_fields = ["username", "password", "host"]
        if db_type == "oracle":
            required_fields.append("service_name")
        else:
            required_fields.append("database")
        if db_type == "postgresql":
            required_fields.append("gssencmode")
        elif db_type == "mssql":
            required_fields.append("charset")
            required_fields.append("tds_version")
        elif db_type == "gbasedbt":
            required_fields.append("server")
        missing_fields = [field for field in required_fields if field not in config]

        if missing_fields:
            raise DriverError(f"缺少必需的连接参数: {', '.join(missing_fields)}")

        # 设置默认端口
        if "port" not in config:
            config["port"] = self._get_default_port(db_type)

        # URL编码敏感信息
        config["username"] = quote_plus(config["username"])
        config["password"] = quote_plus(config["password"])

        # 处理主机名中的特殊字符
        if "@" in config["host"] or ":" in config["host"]:
            config["host"] = quote_plus(config["host"])

        # 构建基础URL
        base_url = self.URL_TEMPLATES[db_type].format(**config)

        logger.debug(f"构建的连接URL: {self._mask_sensitive_info(base_url)}")
        return base_url

    def _get_default_port(self, db_type: str) -> str:
        """
        获取数据库默认端口

        Args:
            db_type: 数据库类型

        Returns:
            str: 默认端口号

        Notes:
            - 如果未配置端口，使用数据库的默认端口
            - 返回空字符串表示该数据库类型无默认端口
        """
        default_ports: Dict[str, str] = {
            "oracle": "1521",
            "postgresql": "5432",
            "mysql": "3306",
            "mssql": "1433",
            "gbasedbt": "9088",
        }
        return default_ports.get(db_type, "")

    def _mask_sensitive_info(self, url: str) -> str:
        """
        掩码连接URL中的敏感信息

        Args:
            url: 原始连接URL

        Returns:
            str: 掩码后的连接URL，密码部分用***替换
        """
        import re

        # 匹配密码部分进行掩码
        return re.sub(r":([^:@]+)@", ":***@", url)

    def _get_pool_config(self) -> Dict[str, Any]:
        """
        获取连接池配置

        Returns:
            Dict[str, Any]: 合并后的连接池配置字典
        """
        # 默认连接池配置
        default_pool_config = {
            "pool_pre_ping": True,  # 连接预检查，确保连接有效
            "pool_recycle": 3600,  # 连接回收时间（秒），避免数据库连接超时
            "pool_size": 10,  # 连接池大小，控制并发连接数
            "max_overflow": 20,  # 最大溢出连接数，应对突发流量
        }

        # 合并用户自定义配置
        user_pool_config = self.connection_config.get("pool_config", {})
        return {**default_pool_config, **user_pool_config}

    def _get_engine_config(self, db_type: str) -> Dict[str, Any]:
        """
        获取数据库特定的引擎配置

        Args:
            db_type: 数据库类型

        Returns:
            Dict[str, Any]: 引擎配置字典

        Notes:
            - 不同数据库类型有不同的配置需求
            - 返回空字典表示无需特殊配置
        """
        config_getters = {
            "oracle": self._get_oracle_config,
            "postgresql": self._get_postgresql_config,
            "mysql": self._get_mysql_config,
            "mssql": self._get_mssql_config,
            "sqlite": self._get_sqlite_config,
            "gbasedbt": self._get_gbase8s_config,
        }

        getter = config_getters.get(db_type)
        return getter() if getter else {}

    def _get_oracle_config(self) -> Dict[str, Any]:
        """
        获取Oracle数据库的引擎配置

        Returns:
            Dict[str, Any]: Oracle引擎配置字典，包含connect_args等参数
        """
        connect_args: Dict[str, Any] = {}
        if "mode" in self.connection_config:
            connect_args["mode"] = self.connection_config["mode"]
        if "threaded" in self.connection_config:
            connect_args["threaded"] = self.connection_config["threaded"]

        return {"connect_args": connect_args} if connect_args else {}

    def _get_postgresql_config(self) -> Dict[str, Any]:
        """
        获取PostgreSQL数据库的引擎配置

        Returns:
            Dict[str, Any]: PostgreSQL引擎配置字典，包含connect_args等参数
        """
        connect_args: Dict[str, Any] = {}
        if "sslmode" in self.connection_config:
            connect_args["sslmode"] = self.connection_config["sslmode"]
        if "sslrootcert" in self.connection_config:
            connect_args["sslrootcert"] = self.connection_config["sslrootcert"]
        if "sslcert" in self.connection_config:
            connect_args["sslcert"] = self.connection_config["sslcert"]
        if "sslkey" in self.connection_config:
            connect_args["sslkey"] = self.connection_config["sslkey"]
        if "connect_timeout" in self.connection_config:
            connect_args["connect_timeout"] = self.connection_config["connect_timeout"]

        return {"connect_args": connect_args} if connect_args else {}

    def _get_mysql_config(self) -> Dict[str, Any]:
        """
        获取MySQL数据库的引擎配置

        Returns:
            Dict[str, Any]: MySQL引擎配置字典，包含connect_args等参数
        """
        connect_args: Dict[str, Any] = {}
        if "charset" in self.connection_config:
            connect_args["charset"] = self.connection_config["charset"]
        if "collation" in self.connection_config:
            connect_args["collation"] = self.connection_config["collation"]
        if "ssl_ca" in self.connection_config:
            connect_args["ssl_ca"] = self.connection_config["ssl_ca"]
        if "ssl_cert" in self.connection_config:
            connect_args["ssl_cert"] = self.connection_config["ssl_cert"]
        if "ssl_key" in self.connection_config:
            connect_args["ssl_key"] = self.connection_config["ssl_key"]

        return {"connect_args": connect_args} if connect_args else {}

    def _get_mssql_config(self) -> Dict[str, Any]:
        """
        获取SQL Server数据库的引擎配置

        Returns:
            Dict[str, Any]: SQL Server引擎配置字典，包含connect_args等参数
        """
        connect_args: Dict[str, Any] = {}
        if "driver" in self.connection_config:
            connect_args["driver"] = self.connection_config["driver"]
        if "trusted_connection" in self.connection_config:
            connect_args["trusted_connection"] = self.connection_config[
                "trusted_connection"
            ]
        if "connect_timeout" in self.connection_config:
            connect_args["connect_timeout"] = self.connection_config["connect_timeout"]

        return {"connect_args": connect_args} if connect_args else {}

    def _get_sqlite_config(self) -> Dict[str, Any]:
        """
        获取SQLite数据库的引擎配置

        Returns:
            Dict[str, Any]: SQLite引擎配置字典，包含connect_args等参数
        """
        connect_args: Dict[str, Any] = {}
        if "timeout" in self.connection_config:
            connect_args["timeout"] = self.connection_config["timeout"]
        if "isolation_level" in self.connection_config:
            connect_args["isolation_level"] = self.connection_config["isolation_level"]
        if "check_same_thread" in self.connection_config:
            connect_args["check_same_thread"] = self.connection_config[
                "check_same_thread"
            ]

        return {"connect_args": connect_args} if connect_args else {}

    def _get_gbase8s_config(self) -> Dict[str, Any]:
        """
        获取GBase 8s数据库的引擎配置

        Returns:
            Dict[str, Any]: GBase 8s引擎配置字典，包含connect_args等参数
        """
        connect_args: Dict[str, Any] = {}
        if "timeout" in self.connection_config:
            connect_args["timeout"] = self.connection_config["timeout"]
        if "isolation_level" in self.connection_config:
            connect_args["isolation_level"] = self.connection_config["isolation_level"]
        if "connect_timeout" in self.connection_config:
            connect_args["connect_timeout"] = self.connection_config["connect_timeout"]

        return {"connect_args": connect_args} if connect_args else {}

    def _validate_connection(self) -> None:
        """
        验证数据库连接的实际有效性

        该方法执行连接测试，如果测试失败则抛出ConnectionError异常。

        Raises:
            ConnectionError: 当连接验证失败时抛出
        """
        if self.engine is None:
            raise ConnectionError("数据库引擎未初始化，无法验证连接")

        if not self._perform_connection_test():
            # 连接验证失败，清理资源
            self._cleanup_on_connection_failure()
            error_msg = "数据库连接验证失败"
            logger.error(error_msg)
            raise ConnectionError(error_msg)

        # 连接验证成功，更新状态
        self._connected = True
        logger.info("数据库连接验证成功...")

    def _perform_connection_test(self) -> bool:
        """
        执行实际的连接测试

        Returns:
            bool: True表示连接测试成功，False表示失败
        """
        if self.engine is None:
            return False

        try:
            test_query = self._get_test_query()
            with self.engine.connect() as conn:
                conn.execute(text(test_query))
            return True
        except Exception:
            return False

    def _get_test_query(self) -> str:
        """
        获取适合当前数据库的测试查询语句

        Returns:
            str: 测试查询语句

        Notes:
            - 不同数据库有不同的测试查询语法
            - Oracle使用DUAL表，其他数据库使用SELECT 1
        """
        db_type = self.connection_config.get("type", "").lower()

        test_queries: Dict[str, str] = {
            "oracle": self.ORACLE_TEST_QUERY,
            "postgresql": self.TEST_QUERY_DEFAULT,
            "mysql": self.TEST_QUERY_DEFAULT,
            "mssql": self.TEST_QUERY_DEFAULT,
            "sqlite": self.TEST_QUERY_DEFAULT,
            "gbasedbt": self.ORACLE_TEST_QUERY,
        }

        return test_queries.get(db_type, self.TEST_QUERY_DEFAULT)

    def _cleanup_on_connection_failure(self) -> None:
        """
        连接失败时的资源清理

        清理已创建的引擎和会话工厂资源，确保资源不会泄漏。
        """
        self._cleanup_resources()
        logger.debug("连接失败，已清理相关资源")

    def _cleanup_resources(self) -> None:
        """
        清理数据库资源

        清理会话工厂和引擎资源，重置连接状态。
        """
        # 清理会话工厂
        if self.session_factory:
            self.session_factory.remove()
            self.session_factory = None

        # 清理引擎资源
        if self.engine:
            self.engine.dispose()
            self.engine = None

        # 重置连接状态
        self._connected = False

    def disconnect(self) -> None:
        """
        断开数据库连接并清理资源

        该方法执行完整的资源清理流程，包括：
        1. 检查当前连接状态，避免重复断开
        2. 清理会话工厂资源
        3. 清理引擎资源
        4. 重置连接状态
        5. 记录断开日志

        Notes:
            - 支持安全断开，即使连接不存在也不会抛出异常
            - 自动清理所有相关资源，避免内存泄漏
            - 线程安全操作，支持并发断开

        Example:
            >>> driver = SQLAlchemyDriver(connection_config)
            >>> driver.connect()
            >>> driver.disconnect()  # 断开数据库连接
            >>> print(driver.is_connected)  # 检查连接状态
            False
        """
        if not self._connected:
            logger.debug("数据库连接已断开，无需重复操作")
            return

        try:
            self._cleanup_resources()
            logger.info("数据库连接已成功断开")
        except Exception as e:
            error_msg = f"数据库连接断开失败: {e.__class__.__name__}: {str(e)}"
            logger.error(error_msg, exc_info=True)
            raise ConnectionError(error_msg)

    def test_connection(self) -> bool:
        """
        测试数据库连接是否有效

        该方法执行连接测试，返回连接状态而不抛出异常。

        Returns:
            bool: True表示连接有效，False表示连接无效

        Notes:
            - 如果当前未连接，会自动尝试建立连接
            - 测试失败不会抛出异常，而是返回False
            - 适合用于连接健康检查或监控

        Example:
            >>> driver = SQLAlchemyDriver(connection_config)
            >>> if driver.test_connection():
            ...     print("连接正常")
            ... else:
            ...     print("连接异常")
        """
        try:
            # 如果未连接，尝试建立连接
            if not self._connected:
                self.connect()
            return self._perform_connection_test()
        except Exception:
            return False

    def execute_query(
        self, query: str, parameters: Dict[str, Any] | None = None
    ) -> List[Dict[str, Any]]:
        """
        执行SQL查询语句并返回结果

        Args:
            query: SQL查询语句
            parameters: 查询参数字典，用于参数化查询

        Returns:
            List[Dict[str, Any]]: 查询结果列表，每行数据为字典格式

        Raises:
            ConnectionError: 当数据库未连接时抛出
            ValueError: 当查询语句为空或仅包含空白字符时抛出
            QueryError: 当查询执行失败时抛出

        Notes:
            - 支持参数化查询，防止SQL注入攻击
            - 自动处理连接状态验证
            - 结果转换为字典格式，便于数据处理
            - 支持事务管理，确保数据一致性

        Example:
            >>> results = driver.execute_query(
            ...     "SELECT * FROM users WHERE age > :age",
            ...     {"age": 18}
            ... )
            >>> for row in results:
            ...     print(row["name"], row["age"])
        """
        if not self._connected or self.engine is None:
            raise ConnectionError("数据库未连接，请先调用connect()方法")

        # 验证查询语句不为空
        if not query or not query.strip():
            raise ValueError("查询语句不能为空")

        try:
            with self.engine.connect() as conn:
                # 执行参数化查询
                result: Result = conn.execute(
                    text(query), parameters if parameters else {}
                )

                # 将结果转换为字典列表
                rows = result.fetchall()
                return [dict(row._mapping) for row in rows]

        except SQLAlchemyError as e:
            error_msg = f"查询执行失败: {e.__class__.__name__}: {str(e)}"
            logger.error(error_msg, exc_info=True)
            raise QueryError(error_msg)

    def execute_command(
        self, command: str, parameters: Dict[str, Any] | None = None
    ) -> int:
        """
        执行SQL命令语句（INSERT/UPDATE/DELETE等）

        Args:
            command: SQL命令语句
            parameters: 命令参数字典，用于参数化执行

        Returns:
            int: 受影响的行数

        Raises:
            ConnectionError: 当数据库未连接时抛出
            ValueError: 当命令语句为空或仅包含空白字符时抛出
            QueryError: 当命令执行失败时抛出

        Notes:
            - 支持参数化执行，防止SQL注入攻击
            - 自动提交事务，确保数据持久化
            - 返回受影响的行数，便于业务逻辑处理

        Example:
            >>> affected_rows = driver.execute_command(
            ...     "UPDATE users SET status = :status WHERE id = :id",
            ...     {"status": "active", "id": 1}
            ... )
            >>> print(f"更新了 {affected_rows} 行数据")
        """
        if not self._connected or self.engine is None:
            raise ConnectionError("数据库未连接，请先调用connect()方法")

        # 验证命令语句不为空
        if not command or not command.strip():
            raise ValueError("命令语句不能为空")

        try:
            with self.engine.begin() as conn:
                # 执行参数化命令
                result: Result = conn.execute(
                    text(command), parameters if parameters else {}
                )
                return result.rowcount

        except SQLAlchemyError as e:
            error_msg = f"命令执行失败: {e.__class__.__name__}: {str(e)}"
            logger.error(error_msg, exc_info=True)
            raise QueryError(error_msg)

    @property
    def is_connected(self) -> bool:
        """
        检查数据库连接状态

        返回当前数据库连接的有效性状态。

        Returns:
            bool: True表示连接有效，False表示连接无效

        Notes:
            - 该属性会执行实际的连接测试，而不仅仅是状态检查
            - 如果连接测试失败，会自动更新内部连接状态
            - 适合用于实时连接状态监控

        Example:
            >>> driver = SQLAlchemyDriver(connection_config)
            >>> driver.connect()
            >>> if driver.is_connected:
            ...     print("连接正常")
            ... else:
            ...     print("连接异常")
        """
        if not self._connected:
            return False

        # 执行实际的连接测试
        is_valid = self._perform_connection_test()
        if not is_valid:
            # 连接测试失败，更新状态
            self._connected = False
            logger.warning("连接状态检查失败，连接已标记为无效")

        return is_valid

    def get_connection_info(self) -> Dict[str, Any]:
        """
        获取数据库连接信息

        返回当前数据库连接的详细信息，包括配置参数和状态。

        Returns:
            Dict[str, Any]: 连接信息字典，包含以下字段：
                - database_type: 数据库类型
                - host: 主机地址
                - port: 端口号
                - database: 数据库名
                - username: 用户名（掩码处理）
                - is_connected: 连接状态
                - pool_size: 连接池大小

        Notes:
            - 敏感信息（如密码）会进行掩码处理
            - 返回完整的连接配置信息，便于调试和监控

        Example:
            >>> info = driver.get_connection_info()
            >>> print(f"连接到 {info['database_type']} 数据库")
            >>> print(f"主机: {info['host']}:{info['port']}")
            >>> print(f"状态: {'已连接' if info['is_connected'] else '未连接'}")
        """
        config = self.connection_config
        pool_config = self._get_pool_config()

        return {
            "database_type": config.get("type"),
            "host": config.get("host"),
            "port": config.get("port", self._get_default_port(config.get("type", ""))),
            "database": config.get("database"),
            "username": config.get("username", "***"),  # 掩码处理
            "is_connected": self._connected,
            "pool_size": pool_config.get("pool_size"),
        }

    def __enter__(self) -> "SQLAlchemyDriver":
        """
        上下文管理器入口方法

        支持with语句，自动管理数据库连接。

        Returns:
            SQLAlchemyDriver: 当前驱动实例

        Notes:
            - 自动建立数据库连接
            - 确保资源正确初始化
            - 支持嵌套上下文管理

        Example:
            >>> with SQLAlchemyDriver(config) as driver:
            ...     results = driver.execute_query("SELECT * FROM users")
        """
        self.connect()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb) -> None:
        """
        上下文管理器退出方法

        自动清理数据库连接资源。

        Args:
            exc_type: 异常类型
            exc_val: 异常值
            exc_tb: 异常追踪信息

        Notes:
            - 无论是否发生异常都会执行资源清理
            - 自动断开数据库连接
            - 确保资源不会泄漏

        Example:
            >>> with SQLAlchemyDriver(config) as driver:
            ...     # 执行数据库操作
            ...     pass
            >>> # 退出with块后连接自动断开
        """
        self.disconnect()

    def __repr__(self) -> str:
        """
        返回驱动实例的字符串表示

        Returns:
            str: 包含数据库类型、主机、端口、数据库名和连接状态的字符串表示

        Example:
            >>> driver = SQLAlchemyDriver(config)
            >>> print(repr(driver))
            SQLAlchemyDriver(type='mysql', host='localhost', port=3306,
                            database='mydb', username='***', connected=True)
        """
        try:
            config = self.connection_config or {}
            db_type = config.get("type", "unknown")
            host = config.get("host", "unknown")
            port = config.get("port", self._get_default_port(db_type))
            database = config.get("database", "unknown")
            username = config.get("username", "***")  # 掩码处理
            status = self._connected

            return (
                f"SQLAlchemyDriver(type={db_type!r}, host={host!r}, "
                f"port={port!r}, database={database!r}, "
                f"username={username!r}, connected={status!r})"
            )
        except Exception:
            # 确保__repr__不会抛出异常
            return "SQLAlchemyDriver(<error retrieving info>)"



"""
DB Connector CLI 工具
====================

提供命令行界面来管理数据库连接和执行查询。

功能特性:
- 支持多种数据库类型 (MySQL, PostgreSQL, Oracle, SQL Server, SQLite, GBase 8s)
- 连接配置管理 (添加、删除、测试、查看)
- SQL查询执行和结果导出
- SQL文件批量执行
- 交互式SQL Shell
- 多种输出格式支持 (表格、JSON、CSV)

使用示例:
    db-connector add mysql-dev --type mysql --host localhost --username root
    db-connector list
    db-connector query mysql-dev "SELECT * FROM users"
    db-connector shell mysql-dev
"""

import argparse
import csv
import json
import os
import re
import sys
from typing import Any, Dict, List, Optional, Tuple, Union

from .core.connections import SUPPORTED_DATABASE_TYPES, DatabaseManager
from .utils.logging_utils import setup_logging

logger = setup_logging(level="debug")


class DBConnectorCLI:
    """
    DB Connector 命令行接口主类

    负责处理所有命令行操作，包括连接管理、查询执行等。

    Attributes:
        db_manager (Optional[DatabaseManager]): 数据库管理器实例
        DB_MANAGER_NOT_INIT_MSG (str): 数据库管理器未初始化的错误消息
        BASIC_PARAMS (List[str]): 基本连接参数列表
    """

    DB_MANAGER_NOT_INIT_MSG = "❌ 数据库管理器未初始化"
    BASIC_PARAMS = [
        "type",
        "host",
        "port",
        "username",
        "password",
        "database",
        "service_name",
        "gssencmode",
        "charset",
        "tds_version",
    ]

    def __init__(self):
        """
        初始化DB Connector CLI

        设置日志系统并准备数据库管理器。
        """
        self.db_manager: Optional[DatabaseManager] = None

    def add_connection(self, args: argparse.Namespace) -> None:
        """
        添加新的数据库连接配置

        Args:
            args (argparse.Namespace): 命令行参数，包含连接配置信息

        Raises:
            SystemExit: 如果添加连接失败则退出程序
        """
        db_manager = self._ensure_db_manager_initialized()
        config = self._build_connection_config(args)

        try:
            db_manager.add_connection(args.name, config)
            logger.info(f"连接 '{args.name}' 添加成功")
            print(f"✅ 连接 '{args.name}' 添加成功")
            self._print_custom_params(config)
        except Exception as e:
            logger.error(f"添加连接失败: {e}")
            print(f"❌ 添加连接失败: {e}")
            sys.exit(1)

    def _ensure_db_manager_initialized(self) -> DatabaseManager:
        """
        确保数据库管理器已初始化

        如果数据库管理器未初始化，则创建新的实例。

        Returns:
            DatabaseManager: 已初始化的数据库管理器实例

        Raises:
            SystemExit: 如果初始化失败则退出程序
        """
        if self.db_manager is None:
            try:
                self.db_manager = DatabaseManager()
            except Exception as e:
                logger.error(f"初始化数据库管理器失败: {e}")
                print(f"❌ 初始化数据库管理器失败: {e}")
                sys.exit(1)
        return self.db_manager

    def _build_connection_config(self, args: argparse.Namespace) -> Dict[str, Any]:
        """
        构建连接配置字典

        Args:
            args (argparse.Namespace): 命令行参数

        Returns:
            Dict[str, Any]: 完整的连接配置字典
        """
        config = {}

        # 添加基本参数
        for param in self.BASIC_PARAMS:
            value = getattr(args, param, None)
            if value is not None:
                config[param] = value

        # 添加自定义参数
        if hasattr(args, "custom_params") and args.custom_params:
            custom_config = self._parse_custom_params(args.custom_params)
            config.update(custom_config)

        return config

    def _parse_custom_params(self, params: List[str]) -> Dict[str, Any]:
        """
        解析自定义参数列表，支持类型自动转换

        Args:
            params (List[str]): 参数字符串列表，格式为 key=value

        Returns:
            Dict[str, Any]: 转换后的键值对字典

        Raises:
            ValueError: 如果参数格式无效

        Example:
            >>> cli = DBConnectorCLI()
            >>> cli._parse_custom_params(["timeout=30", "ssl=true"])
            {'timeout': 30, 'ssl': True}
        """
        result = {}
        for param in params:
            if "=" not in param:
                logger.warning(f"忽略无效的自定义参数格式: {param}")
                continue

            key, value = param.split("=", 1)
            key = key.strip()
            value = value.strip()

            if not key:
                logger.warning(f"忽略空键名的参数: {param}")
                continue

            result[key] = self._convert_value_type(value)

        return result

    def _convert_value_type(self, value: str) -> Union[str, int, float, bool]:
        """
        智能转换参数值的数据类型

        支持转换: 布尔值(true/false)、整数、浮点数、字符串

        Args:
            value (str): 原始字符串值

        Returns:
            Union[str, int, float, bool]: 转换后的值

        Example:
            >>> cli = DBConnectorCLI()
            >>> cli._convert_value_type("true")
            True
            >>> cli._convert_value_type("123")
            123
            >>> cli._convert_value_type("3.14")
            3.14
            >>> cli._convert_value_type("hello")
            'hello'
        """
        value_lower = value.lower().strip()

        # 布尔值转换
        if value_lower in ("true", "false"):
            return value_lower == "true"

        # 整数转换
        if value.isdigit():
            return int(value)

        # 浮点数转换
        try:
            return float(value)
        except ValueError:
            pass

        # 保持原字符串
        return value

    def _print_custom_params(self, config: Dict[str, Any]) -> None:
        """
        打印自定义参数信息

        Args:
            config (Dict[str, Any]): 连接配置字典
        """
        custom_params = [k for k in config.keys() if k not in self.BASIC_PARAMS]
        if custom_params:
            print(f"📋 自定义参数: {', '.join(custom_params)}")

    def remove_connection(self, args: argparse.Namespace) -> None:
        """
        删除指定的数据库连接配置

        Args:
            args (argparse.Namespace): 命令行参数，包含要删除的连接名称

        Raises:
            SystemExit: 如果删除连接失败则退出程序
        """
        db_manager = self._ensure_db_manager_initialized()

        try:
            db_manager.remove_connection(args.name)
            logger.info(f"连接 '{args.name}' 已删除")
            print(f"✅ 连接 '{args.name}' 已删除")
        except Exception as e:
            logger.error(f"删除连接失败: {e}")
            print(f"❌ 删除连接失败: {e}")
            sys.exit(1)

    def update_connection(self, args: argparse.Namespace) -> None:
        """
        更新数据库连接配置

        Args:
            args (argparse.Namespace): 命令行参数，包含连接名称和更新配置

        Raises:
            SystemExit: 如果更新连接失败则退出程序
        """
        db_manager = self._ensure_db_manager_initialized()

        try:
            existing_config = db_manager.show_connection(args.name)
            update_config = self._build_update_config(existing_config, args)

            db_manager.update_connection(args.name, update_config)
            logger.info(f"连接 '{args.name}' 更新成功")
            print(f"✅ 连接 '{args.name}' 更新成功")
            self._print_custom_params(update_config)
        except Exception as e:
            logger.error(f"更新连接失败: {e}")
            print(f"❌ 更新连接失败: {e}")
            sys.exit(1)

    def _build_update_config(
        self, existing_config: Dict[str, Any], args: argparse.Namespace
    ) -> Dict[str, Any]:
        """
        根据命令行参数构建更新后的配置

        Args:
            existing_config (Dict[str, Any]): 现有配置
            args (argparse.Namespace): 命令行参数

        Returns:
            Dict[str, Any]: 更新后的配置
        """
        update_config = existing_config.copy()

        # 更新基本参数
        for param in self.BASIC_PARAMS:
            value = getattr(args, param, None)
            if value is not None:
                update_config[param] = value

        # 更新自定义参数
        if hasattr(args, "custom_params") and args.custom_params:
            custom_config = self._parse_custom_params(args.custom_params)
            update_config.update(custom_config)

        return update_config

    def show_connection(self, args: argparse.Namespace) -> None:
        """
        显示指定连接的详细配置信息

        注意：敏感信息（如密码）会被隐藏显示。

        Args:
            args (argparse.Namespace): 命令行参数，包含要显示详情的连接名称

        Raises:
            SystemExit: 如果获取连接详情失败则退出程序
        """
        db_manager = self._ensure_db_manager_initialized()

        try:
            config = db_manager.show_connection(args.name)
            safe_config = self._sanitize_sensitive_info(config)

            print(f"🔍 连接 '{args.name}' 的配置:")
            self._display_connection_details(safe_config)

        except Exception as e:
            logger.error(f"获取连接详情失败: {e}")
            print(f"❌ 获取连接详情失败: {e}")
            sys.exit(1)

    def _sanitize_sensitive_info(self, config: Dict[str, Any]) -> Dict[str, Any]:
        """
        隐藏敏感信息（如密码）

        Args:
            config (Dict[str, Any]): 原始配置

        Returns:
            Dict[str, Any]: 安全配置（敏感信息已隐藏）
        """
        safe_config = config.copy()
        password_fields = ["password", "passwd", "pwd"]

        for field in password_fields:
            if field in safe_config:
                safe_config[field] = "***"

        return safe_config

    def _display_connection_details(self, config: Dict[str, Any]) -> None:
        """
        显示连接配置详情

        Args:
            config (Dict[str, Any]): 连接配置
        """
        # 显示基本参数
        for key in self.BASIC_PARAMS:
            if key in config:
                print(f"  {key}: {config[key]}")

        # 显示自定义参数
        custom_params = [k for k in config.keys() if k not in self.BASIC_PARAMS]
        if custom_params:
            print("\n  📋 自定义参数:")
            for key in custom_params:
                print(f"    {key}: {config[key]}")

    def list_connections(self, _args: argparse.Namespace) -> None:
        """
        列出所有已配置的数据库连接

        Args:
            _args (argparse.Namespace): 命令行参数（未使用）

        Raises:
            SystemExit: 如果列出连接失败则退出程序
        """
        db_manager = self._ensure_db_manager_initialized()

        try:
            connections = db_manager.list_connections()
            if connections:
                print("📋 已配置的连接:")
                for i, conn in enumerate(connections, 1):
                    print(f"  {i}. {conn}")
            else:
                print("ℹ️  没有配置任何连接")
        except Exception as e:
            logger.error(f"列出连接失败: {e}")
            print(f"❌ 列出连接失败: {e}")
            sys.exit(1)

    def test_connection(self, args: argparse.Namespace) -> None:
        """
        测试指定连接的连通性

        Args:
            args (argparse.Namespace): 命令行参数，包含要测试的连接名称

        Raises:
            SystemExit: 如果连接测试失败则退出程序
        """
        db_manager = self._ensure_db_manager_initialized()

        try:
            if db_manager.test_connection(args.name):
                print(f"✅ 连接 '{args.name}' 测试成功")
            else:
                print(f"❌ 连接 '{args.name}' 测试失败")
                sys.exit(1)
        except Exception as e:
            logger.error(f"连接测试失败: {e}")
            print(f"❌ 连接测试失败: {e}")
            sys.exit(1)

    def execute_query(self, args: argparse.Namespace) -> None:
        """
        在指定连接上执行SQL查询

        Args:
            args: 命令行参数，包含连接名称、查询语句和输出选项

        Raises:
            SystemExit: 如果执行查询失败则退出程序
        """
        db_manager = self._ensure_db_manager_initialized()

        try:
            results = db_manager.execute_query(args.connection, args.query)

            if args.output:
                self._save_output(results, args.output, args.format)
            else:
                self._display_results(results, args.format)

        except Exception as e:
            logger.error(f"执行查询失败: {e}")
            print(f"❌ 执行查询失败: {e}")
            sys.exit(1)

    def _save_output(self, results: List[Dict], output_path: str, format: str) -> None:
        """
        将查询结果保存到文件

        Args:
            results: 查询结果列表
            output_path: 输出文件路径
            format: 输出格式

        Raises:
            SystemExit: 如果保存失败
        """
        try:
            if format == "json":
                with open(output_path, "w", encoding="utf-8") as f:
                    json.dump(results, f, indent=2, ensure_ascii=False)
            elif format == "csv":
                with open(output_path, "w", newline="", encoding="utf-8") as f:
                    if results:
                        headers = list(results[0].keys())
                        writer = csv.DictWriter(f, fieldnames=headers)
                        writer.writeheader()
                        writer.writerows(results)
            else:  # table格式保存为文本
                with open(output_path, "w", encoding="utf-8") as f:
                    # 重定向标准输出到文件
                    original_stdout = sys.stdout
                    sys.stdout = f
                    self._display_table(results)
                    sys.stdout = original_stdout

            print(f"✅ 结果已保存到: {output_path}")
        except Exception as e:
            logger.error(f"保存结果失败: {e}")
            print(f"❌ 保存结果失败: {e}")
            sys.exit(1)

    def _display_table(self, results: List[Dict]) -> None:
        """
        以表格形式显示查询结果

        Args:
            results: 查询结果列表
        """
        if not results:
            return

        headers = list(results[0].keys())

        # 计算每列的最大宽度
        col_widths = {}
        for header in headers:
            col_widths[header] = len(str(header))

        for row in results:
            for header in headers:
                value = str(row.get(header, ""))
                col_widths[header] = max(col_widths[header], len(value))

        # 限制最大列宽
        max_col_width = 50
        for header in headers:
            col_widths[header] = min(col_widths[header], max_col_width)

        # 打印表头
        header_line = " | ".join(
            [f"{header:<{col_widths[header]}}" for header in headers]
        )
        separator = "-+-".join(["-" * col_widths[header] for header in headers])

        print(separator)
        print(header_line)
        print(separator)

        # 打印数据行
        for row in results:
            row_line = " | ".join(
                [
                    f"{self._truncate_value(str(row.get(header, '')), col_widths[header]):<{col_widths[header]}}"
                    for header in headers
                ]
            )
            print(row_line)

        print(separator)
        print(f"总计: {len(results)} 行")

    def _truncate_value(self, value: str, max_length: int) -> str:
        """
        截断过长的值用于表格显示

        Args:
            value: 原始值
            max_length: 最大长度

        Returns:
            str: 截断后的值
        """
        if len(value) <= max_length:
            return value
        return value[: max_length - 3] + "..."

    def _display_results(self, results: List[Dict], format: str = "table") -> None:
        """
        以指定格式显示查询结果

        Args:
            results: 查询结果列表
            format: 显示格式 (table/json/csv)

        Raises:
            ValueError: 如果格式不支持
        """
        if not results:
            print("没有结果")
            return

        if format == "table":
            self._display_table(results)
        elif format == "json":
            self._display_json(results)
        elif format == "csv":
            self._display_csv(results)
        else:
            print(f"❌ 不支持的输出格式: {format}")
            print("✅ 支持的格式: table, json, csv")
            sys.exit(1)

    def _display_json(self, results: List[Dict]) -> None:
        """
        以JSON格式显示查询结果

        Args:
            results: 查询结果列表
        """
        try:
            print(json.dumps(results, indent=2, ensure_ascii=False))
        except Exception as e:
            logger.error(f"JSON序列化失败: {e}")
            print(f"❌ JSON序列化失败: {e}")
            sys.exit(1)

    def _display_csv(self, results: List[Dict]) -> None:
        """
        以CSV格式显示查询结果

        Args:
            results: 查询结果列表
        """
        if not results:
            return

        headers = list(results[0].keys())
        writer = csv.DictWriter(sys.stdout, fieldnames=headers)
        writer.writeheader()
        for row in results:
            writer.writerow(row)

    def execute_file(self, args: argparse.Namespace) -> None:
        """
        执行SQL文件中的多个语句

        支持批量执行SQL文件，可选择在遇到错误时继续执行。

        Args:
            args: 命令行参数，包含连接名称、文件路径和执行选项

        Raises:
            SystemExit: 如果执行文件失败则退出程序
        """
        db_manager = self._ensure_db_manager_initialized()

        # 验证文件存在性
        if not os.path.exists(args.file):
            logger.error(f"SQL文件不存在: {args.file}")
            print(f"❌ SQL文件不存在: {args.file}")
            sys.exit(1)

        try:
            statements = self._read_and_split_sql_file(args.file)
            if not statements:
                print("ℹ️  SQL文件中没有有效的SQL语句")
                return

            results, success_count, error_count = self._execute_sql_statements(
                db_manager, statements, args.connection, args.continue_on_error
            )

            self._print_execution_summary(success_count, error_count)

            if results and args.output:
                self._save_output(results, args.output, args.format)
            elif results:
                self._display_results(results, args.format)

        except Exception as e:
            logger.error(f"执行SQL文件失败: {e}")
            print(f"❌ 执行SQL文件失败: {e}")
            sys.exit(1)

    def _read_and_split_sql_file(self, file_path: str) -> List[str]:
        """
        读取SQL文件内容并分割为独立的SQL语句

        Args:
            file_path: SQL文件路径

        Returns:
            List[str]: 分割后的SQL语句列表

        Raises:
            UnicodeDecodeError: 如果文件编码不支持
        """
        try:
            with open(file_path, "r", encoding="utf-8") as f:
                sql_content = f.read()
            return self._split_sql_statements(sql_content)
        except UnicodeDecodeError:
            # 尝试其他编码
            try:
                with open(file_path, "r", encoding="gbk") as f:
                    sql_content = f.read()
                return self._split_sql_statements(sql_content)
            except UnicodeDecodeError as e:
                logger.error(f"无法解码SQL文件: {e}")
                print("❌ 无法解码SQL文件，请检查文件编码")
                sys.exit(1)

    def _split_sql_statements(self, sql_content: str) -> List[str]:
        """
        将SQL内容分割为独立的语句

        支持处理多行注释和复杂语句。

        Args:
            sql_content: 原始SQL内容

        Returns:
            List[str]: 分割后的SQL语句列表
        """
        # 移除多行注释
        sql_content = re.sub(r"/\*.*?\*/", "", sql_content, flags=re.DOTALL)

        # 按分号分割语句
        statements = []
        current_statement = ""

        for line in sql_content.split("\n"):
            line = line.strip()
            if line.startswith("--"):  # 跳过单行注释
                continue
            current_statement += " " + line
            if line.endswith(";"):
                statements.append(current_statement.strip())
                current_statement = ""

        # 处理最后可能没有分号的语句
        if current_statement.strip():
            statements.append(current_statement.strip())

        return [stmt for stmt in statements if stmt and not stmt.isspace()]

    def _execute_sql_statements(
        self,
        db_manager: DatabaseManager,
        statements: List[str],
        connection_name: str,
        continue_on_error: bool,
    ) -> Tuple[List[Dict], int, int]:
        """
        执行SQL语句列表

        Args:
            db_manager: 数据库管理器实例
            statements: SQL语句列表
            connection_name: 连接名称
            continue_on_error: 遇到错误时是否继续执行

        Returns:
            tuple: (查询结果列表, 成功执行数, 失败执行数)
        """
        total_results = []
        success_count = 0
        error_count = 0

        for i, statement in enumerate(statements, 1):
            if not statement.strip():
                continue

            print(f"执行语句 {i}/{len(statements)}: {self._truncate_sql(statement)}")

            try:
                if statement.lower().strip().startswith("select"):
                    results = db_manager.execute_query(connection_name, statement)
                    total_results.extend(results)
                    success_count += 1
                    print(f"✅ 查询成功，返回 {len(results)} 行结果")
                else:
                    affected = db_manager.execute_command(connection_name, statement)
                    print(f"✅ 执行成功，影响行数: {affected}")
                    success_count += 1
            except Exception as e:
                error_count += 1
                logger.error(f"执行语句失败: {e}")
                print(f"❌ 执行语句失败: {e}")
                if not continue_on_error:
                    sys.exit(1)

        return total_results, success_count, error_count

    def _truncate_sql(self, sql: str, max_length: int = 50) -> str:
        """
        截断SQL语句用于显示，避免过长的输出

        Args:
            sql: 原始SQL语句
            max_length: 最大显示长度

        Returns:
            str: 截断后的SQL语句
        """
        if len(sql) <= max_length:
            return sql
        return sql[:max_length] + "..."

    def _print_execution_summary(self, success_count: int, error_count: int) -> None:
        """
        打印SQL执行统计信息

        Args:
            success_count: 成功执行的语句数
            error_count: 执行失败的语句数
        """
        total = success_count + error_count
        success_rate = (success_count / total * 100) if total > 0 else 0
        print(f"\n执行完成: 成功 {success_count} 条，失败 {error_count} 条")
        print(f"成功率: {success_rate:.1f}%")

    def interactive_shell(self, args: argparse.Namespace) -> None:
        """
        启动交互式SQL Shell

        Args:
            args: 命令行参数，包含连接名称

        Raises:
            SystemExit: 如果启动Shell失败
        """
        db_manager = self._ensure_db_manager_initialized()

        try:
            print(f"🚀 启动SQL Shell (连接: {args.connection})")
            print("输入 'exit' 或 'quit' 退出")
            print("输入 'help' 查看帮助")

            while True:
                try:
                    sql = input(f"{args.connection}> ").strip()
                    if sql.lower() in ["exit", "quit"]:
                        break
                    elif sql.lower() == "help":
                        self._print_shell_help()
                        continue
                    elif not sql:
                        continue

                    # 执行SQL
                    if sql.lower().startswith("select"):
                        results = db_manager.execute_query(args.connection, sql)
                        self._display_results(results, "table")
                    else:
                        affected = db_manager.execute_command(args.connection, sql)
                        print(f"影响行数: {affected}")

                except KeyboardInterrupt:
                    print("\n👋 再见!")
                    break
                except Exception as e:
                    print(f"❌ 执行错误: {e}")

        except Exception as e:
            logger.error(f"启动SQL Shell失败: {e}")
            print(f"❌ 启动SQL Shell失败: {e}")
            sys.exit(1)

    def _print_shell_help(self) -> None:
        """打印SQL Shell帮助信息"""
        help_text = """
SQL Shell 命令:
  exit, quit    - 退出Shell
  help         - 显示此帮助信息

支持的SQL语句:
  SELECT       - 执行查询并显示结果
  INSERT       - 插入数据
  UPDATE       - 更新数据
  DELETE       - 删除数据
  CREATE       - 创建表或数据库
  DROP         - 删除表或数据库
        """
        print(help_text)


class ChineseHelpFormatter(argparse.RawDescriptionHelpFormatter):
    """中文帮助格式化器，优化帮助信息显示"""

    def _format_usage(self, usage, actions, groups, prefix):
        """格式化使用说明"""
        if prefix is None:
            prefix = "\n使用情况: "
        return super()._format_usage(usage, actions, groups, prefix)

    def start_section(self, heading):
        """开始新的帮助章节"""
        if heading == "options":
            heading = "下列选项可用"
        super().start_section(heading)


def create_argument_parser(cli_instance: DBConnectorCLI) -> argparse.ArgumentParser:
    """
    创建命令行参数解析器

    Args:
        cli_instance (DBConnectorCLI): 已初始化的CLI实例

    Returns:
        argparse.ArgumentParser: 配置好的参数解析器
    """
    parser = argparse.ArgumentParser(
        usage="db-connector [<命令>] [<选项>]",
        description="DB Connector - 数据库连接管理工具",
        formatter_class=ChineseHelpFormatter,
        epilog="""
使用示例:
  db-connector add mysql-dev --type mysql --host localhost --username root
  db-connector list
  db-connector query mysql-dev "SELECT * FROM users"
  db-connector shell mysql-dev

更多帮助请参考项目文档。
        """,
        add_help=False,
    )

    # 添加自定义的帮助选项
    parser.add_argument(
        "-h",
        "--help",
        action="help",
        default=argparse.SUPPRESS,
        help="显示选定命令的帮助信息",
    )

    subparsers = parser.add_subparsers(title="下列命令有效", dest="command")

    # add 命令
    add_parser = subparsers.add_parser("add", help="添加新的数据库连接")
    _setup_connection_arguments(add_parser)
    add_parser.set_defaults(func=cli_instance.add_connection)

    # remove 命令
    remove_parser = subparsers.add_parser("remove", help="删除连接")
    remove_parser.add_argument("name", help="连接名称")
    remove_parser.set_defaults(func=cli_instance.remove_connection)

    # update 命令
    update_parser = subparsers.add_parser("update", help="更新连接配置")
    _setup_connection_arguments(update_parser)
    update_parser.set_defaults(func=cli_instance.update_connection)

    # show 命令
    show_parser = subparsers.add_parser("show", help="显示连接详情")
    show_parser.add_argument("name", help="连接名称")
    show_parser.set_defaults(func=cli_instance.show_connection)

    # list 命令
    list_parser = subparsers.add_parser("list", help="列出所有连接")
    list_parser.set_defaults(func=cli_instance.list_connections)

    # test 命令
    test_parser = subparsers.add_parser("test", help="测试连接")
    test_parser.add_argument("name", help="连接名称")
    test_parser.set_defaults(func=cli_instance.test_connection)

    # query 命令
    query_parser = subparsers.add_parser("query", help="执行SQL查询")
    query_parser.add_argument("connection", help="连接名称")
    query_parser.add_argument("query", help="SQL查询语句")
    query_parser.add_argument(
        "--format",
        choices=["table", "json", "csv"],
        default="table",
        help="输出格式 (默认: table)",
    )
    query_parser.add_argument("--output", help="输出文件路径")
    query_parser.set_defaults(func=cli_instance.execute_query)

    # execute 命令
    execute_parser = subparsers.add_parser("execute", help="执行SQL文件")
    execute_parser.add_argument("connection", help="连接名称")
    execute_parser.add_argument("file", help="SQL文件路径")
    execute_parser.add_argument(
        "--format",
        choices=["table", "json", "csv"],
        default="table",
        help="输出格式 (默认: table)",
    )
    execute_parser.add_argument("--output", help="输出文件路径")
    execute_parser.add_argument(
        "--continue-on-error", action="store_true", help="遇到错误时继续执行"
    )
    execute_parser.set_defaults(func=cli_instance.execute_file)

    # shell 命令
    shell_parser = subparsers.add_parser("shell", help="启动交互式SQL Shell")
    shell_parser.add_argument("connection", help="连接名称")
    shell_parser.set_defaults(func=cli_instance.interactive_shell)

    return parser


def _setup_connection_arguments(parser: argparse.ArgumentParser) -> None:
    """
    设置连接相关的命令行参数

    Args:
        parser (argparse.ArgumentParser): 参数解析器实例
    """
    parser.add_argument("name", help="连接名称")
    parser.add_argument(
        "-T",
        "--type",
        required=True,
        choices=SUPPORTED_DATABASE_TYPES,
        help="数据库类型",
    )
    parser.add_argument("-H", "--host", help="数据库主机")
    parser.add_argument("-P", "--port", type=int, help="数据库端口")
    parser.add_argument("-u", "--username", help="用户名")
    parser.add_argument("-p", "--password", help="密码")
    parser.add_argument("-d", "--database", help="数据库名")
    parser.add_argument("-s", "--service-name", help="Oracle服务名称")
    parser.add_argument(
        "-g",
        "--gssencmode",
        choices=["disable", "allow", "prefer", "require"],
        help="PostgreSQL GSSENCMODE 参数",
    )
    parser.add_argument(
        "-k",
        "--charset",
        choices=["cp936", "utf8", "utf8mb4"],
        help="MySQL/MSSQL字符集",
    )
    parser.add_argument(
        "-t",
        "--tds-version",
        choices=["7.0", "7.1", "7.2", "7.3", "7.4", "8.0"],
        help="MSSQL TDS 版本",
    )
    parser.add_argument(
        "-c", "--custom-params", nargs="+", help="自定义参数 (格式: key=value)"
    )


def main():
    """DB Connector CLI 主入口函数"""
    cli = DBConnectorCLI()
    parser = create_argument_parser(cli)

    if len(sys.argv) == 1:
        parser.print_help()
        sys.exit(0)

    args = parser.parse_args()

    if hasattr(args, "func"):
        args.func(args)
    else:
        parser.print_help()


if __name__ == "__main__":
    main()
